@@ -12,6 +12,7 @@ EAPI Ecore_X_Atom E_ATOM_BORDER_SIZE = 0;
 EAPI Ecore_X_Atom E_ATOM_WINDOW_STATE = 0;
 EAPI Ecore_X_Atom E_ATOM_WINDOW_STATE_CENTERED = 0;
 EAPI Ecore_X_Atom E_ATOM_DESKTOP_FILE = 0;
+EAPI Ecore_X_Atom E_ATOM_ZONE_GEOMETRY = 0;
 
 /* externally accessible functions */
 EINTERN int
@@ -28,11 +29,12 @@ e_atoms_init(void)
 	"__E_WINDOW_BORDER_SIZE",
 	"__E_ATOM_WINDOW_STATE",
         "__E_ATOM_WINDOW_STATE_CENTERED",
-        "__E_ATOM_DESKTOP_FILE"
+        "__E_ATOM_DESKTOP_FILE",
+	"E_ZONE_GEOMETRY"
    };
-   Ecore_X_Atom atoms[11];
-   
-   ecore_x_atoms_get(atom_names, 11, atoms);
+   Ecore_X_Atom atoms[12];
+
+   ecore_x_atoms_get(atom_names, 12, atoms);
    E_ATOM_MANAGED = atoms[0];
    E_ATOM_CONTAINER = atoms[1];
    E_ATOM_ZONE = atoms[2];
@@ -44,6 +46,7 @@ e_atoms_init(void)
    E_ATOM_WINDOW_STATE = atoms[8];
    E_ATOM_WINDOW_STATE_CENTERED = atoms[9];
    E_ATOM_DESKTOP_FILE = atoms[10];
+   E_ATOM_ZONE_GEOMETRY = atoms[11];
    return 1;
 }
 
