@@ -20,22 +20,22 @@ _e_dialog_internal_new(E_Container *con, const char *name, const char *class, in
    Evas_Object *o;
    Evas_Modifier_Mask mask;
    Eina_Bool kg;
-   
+
    if (!con)
      {
-	man = e_manager_current_get();
-	if (!man) return NULL;
-	con = e_container_current_get(man);
-	if (!con) con = e_container_number_get(man, 0);
-	if (!con) return NULL;
+        man = e_manager_current_get();
+        if (!man) return NULL;
+        con = e_container_current_get(man);
+        if (!con) con = e_container_number_get(man, 0);
+        if (!con) return NULL;
      }
    dia = E_OBJECT_ALLOC(E_Dialog, E_DIALOG_TYPE, _e_dialog_free);
    if (!dia) return NULL;
    dia->win = e_win_new(con);
    if (!dia->win)
      {
-	free(dia);
-	return NULL;
+        free(dia);
+        return NULL;
      }
    e_win_delete_callback_set(dia->win, _e_dialog_cb_delete);
    e_win_resize_callback_set(dia->win, _e_dialog_cb_resize);
@@ -45,7 +45,7 @@ _e_dialog_internal_new(E_Container *con, const char *name, const char *class, in
    o = edje_object_add(e_win_evas_get(dia->win));
    dia->bg_object = o;
    e_theme_edje_object_set(o, "base/theme/dialog",
-			   "e/widgets/dialog/main");
+                           "e/widgets/dialog/main");
    evas_object_move(o, 0, 0);
    evas_object_show(o);
 
@@ -53,25 +53,30 @@ _e_dialog_internal_new(E_Container *con, const char *name, const char *class, in
    e_widget_on_focus_hook_set(o, _e_dialog_cb_wid_on_focus, dia);
    dia->box_object = o;
    edje_object_part_swallow(dia->bg_object, "e.swallow.buttons", o);
+   edje_object_signal_emit(dia->bg_object, "e,state,no_resizeble", "e");
 
    o = evas_object_rectangle_add(e_win_evas_get(dia->win));
    dia->event_object = o;
    mask = 0;
    kg = evas_object_key_grab(o, "Tab", mask, ~mask, 0);
    if (!kg)
-      fprintf(stderr,"ERROR: unable to redirect \"Tab\" key events to object %p.\n", o);
+     fprintf(stderr, "ERROR: unable to redirect \"Tab\" key events to object %p.\n", o);
    mask = evas_key_modifier_mask_get(e_win_evas_get(dia->win), "Shift");
    kg = evas_object_key_grab(o, "Tab", mask, ~mask, 0);
    if (!kg)
-      fprintf(stderr,"ERROR: unable to redirect \"Tab\" key events to object %p.\n", o);
+     fprintf(stderr, "ERROR: unable to redirect \"Tab\" key events to object %p.\n", o);
    mask = 0;
    kg = evas_object_key_grab(o, "Return", mask, ~mask, 0);
    if (!kg)
-      fprintf(stderr,"ERROR: unable to redirect \"Return\" key events to object %p.\n", o);
+     fprintf(stderr, "ERROR: unable to redirect \"Return\" key events to object %p.\n", o);
+   mask = 0;
+   kg = evas_object_key_grab(o, "Escape", mask, ~mask, 0);
+   if (!kg)
+     fprintf(stderr, "ERROR: unable to redirect \"Escape\" key events to object %p.\n", o);
    mask = 0;
    kg = evas_object_key_grab(o, "KP_Enter", mask, ~mask, 0);
    if (!kg)
-      fprintf(stderr,"ERROR: unable to redirect \"KP_Enter\" key events to object %p.\n", o);
+     fprintf(stderr, "ERROR: unable to redirect \"KP_Enter\" key events to object %p.\n", o);
 
    evas_object_event_callback_add(o, EVAS_CALLBACK_KEY_DOWN, _e_dialog_cb_key_down, dia);
 
@@ -91,12 +96,12 @@ e_dialog_normal_win_new(E_Container *con, const char *name, const char *class)
 }
 
 EAPI void
-e_dialog_button_add(E_Dialog *dia, const char *label, const char *icon, void (*func) (void *data, E_Dialog *dia), void *data)
+e_dialog_button_add(E_Dialog *dia, const char *label, const char *icon, E_Dialog_Cb func, void *data)
 {
    Evas_Object *o;
 
    if (!func) func = _e_dialog_del_func_cb;
-   o = e_widget_button_add(e_win_evas_get(dia->win), label, icon, (void (*) (void*, void*)) func, data, dia);
+   o = e_widget_button_add(e_win_evas_get(dia->win), label, icon, (void (*)(void *, void *))func, data, dia);
    e_widget_list_object_append(dia->box_object, o, 1, 0, 0.5);
    dia->buttons = eina_list_append(dia->buttons, o);
 }
@@ -105,7 +110,7 @@ EAPI int
 e_dialog_button_focus_num(E_Dialog *dia, int button)
 {
    Evas_Object *o;
-   
+
    o = eina_list_nth(dia->buttons, button);
    if (o) e_widget_focus_steal(o);
    return 1;
@@ -115,7 +120,7 @@ EAPI int
 e_dialog_button_disable_num_set(E_Dialog *dia, int button, int disabled)
 {
    Evas_Object *o;
-   
+
    o = eina_list_nth(dia->buttons, button);
    if (o) e_widget_disabled_set(o, disabled);
    return 1;
@@ -126,7 +131,7 @@ e_dialog_button_disable_num_get(E_Dialog *dia, int button)
 {
    Evas_Object *o;
    int ret = 0;
-   
+
    o = eina_list_nth(dia->buttons, button);
    if (o) ret = e_widget_disabled_get(o);
    return ret;
@@ -143,14 +148,14 @@ e_dialog_text_set(E_Dialog *dia, const char *text)
 {
    if (!dia->text_object)
      {
-	Evas_Object *o;
-	
-	o = edje_object_add(e_win_evas_get(dia->win));
-	dia->text_object = o;
-	e_theme_edje_object_set(o, "base/theme/dialog",
-				"e/widgets/dialog/text");
-	edje_object_part_swallow(dia->bg_object, "e.swallow.content", o);
-	evas_object_show(o);
+        Evas_Object *o;
+
+        o = edje_object_add(e_win_evas_get(dia->win));
+        dia->text_object = o;
+        e_theme_edje_object_set(o, "base/theme/dialog",
+                                "e/widgets/dialog/text");
+        edje_object_part_swallow(dia->bg_object, "e.swallow.content", o);
+        evas_object_show(o);
      }
    edje_object_part_text_set(dia->text_object, "e.textblock.message", text);
 }
@@ -161,24 +166,30 @@ e_dialog_icon_set(E_Dialog *dia, const char *icon, Evas_Coord size)
    if (!icon) return;
 
    dia->icon_object = e_icon_add(e_win_evas_get(dia->win));
-   e_util_icon_theme_set(dia->icon_object, icon);
+   if (!e_util_icon_theme_set(dia->icon_object, icon))
+     e_icon_file_edje_set(dia->icon_object, icon, "icon");
    edje_extern_object_min_size_set(dia->icon_object, size * e_scale, size * e_scale);
    edje_object_part_swallow(dia->bg_object, "e.swallow.icon", dia->icon_object);
    edje_object_signal_emit(dia->bg_object, "e,state,icon", "e");
    evas_object_show(dia->icon_object);
+   if (icon)
+     edje_object_signal_emit(dia->bg_object, "e,icon,enabled", "e");
+   else
+     edje_object_signal_emit(dia->bg_object, "e,icon,disabled", "e");
+   edje_object_message_signal_process(dia->bg_object);
 }
 
-EAPI void 
-e_dialog_border_icon_set(E_Dialog *dia, const char *icon) 
+EAPI void
+e_dialog_border_icon_set(E_Dialog *dia, const char *icon)
 {
    E_Border *border;
-   
+
    border = dia->win->border;
    if (!border) return;
    if (border->internal_icon)
      {
-	eina_stringshare_del(border->internal_icon);
-	border->internal_icon = NULL;
+        eina_stringshare_del(border->internal_icon);
+        border->internal_icon = NULL;
      }
    if (icon)
      border->internal_icon = eina_stringshare_add(icon);
@@ -187,10 +198,16 @@ e_dialog_border_icon_set(E_Dialog *dia, const char *icon)
 EAPI void
 e_dialog_content_set(E_Dialog *dia, Evas_Object *obj, Evas_Coord minw, Evas_Coord minh)
 {
+   int mw, mh;
    dia->content_object = obj;
    e_widget_on_focus_hook_set(obj, _e_dialog_cb_wid_on_focus, dia);
    edje_extern_object_min_size_set(obj, minw, minh);
    edje_object_part_swallow(dia->bg_object, "e.swallow.content", obj);
+   edje_object_size_min_calc(dia->bg_object, &mw, &mh);
+   e_win_resize(dia->win, mw, mh);
+   e_win_size_min_set(dia->win, mw, mh);
+   dia->min_w = mw;
+   dia->min_h = mh;
    evas_object_show(obj);
 }
 
@@ -200,40 +217,51 @@ e_dialog_resizable_set(E_Dialog *dia, int resizable)
    dia->resizable = resizable;
    if (dia->win)
      {
-	if (resizable)
-	  {
-	     e_win_size_max_set(dia->win, 99999, 99999);
+        if (resizable)
+          {
+             e_win_size_max_set(dia->win, 99999, 99999);
              e_util_win_auto_resize_fill(dia->win);
-	     edje_object_signal_emit(dia->bg_object, "e,state,resizeble", "e");
-	  }
-	else
-	  {
-	     e_win_resize(dia->win, dia->min_w, dia->min_h);
-	     e_win_size_max_set(dia->win, dia->min_w, dia->min_h);
-	     edje_object_signal_emit(dia->bg_object, "e,state,no_resizeble", "e");
-	  }
+             edje_object_signal_emit(dia->bg_object, "e,state,resizeble", "e");
+          }
+        else
+          {
+             e_win_resize(dia->win, dia->min_w, dia->min_h);
+             e_win_size_max_set(dia->win, dia->min_w, dia->min_h);
+             edje_object_signal_emit(dia->bg_object, "e,state,no_resizeble", "e");
+          }
      }
 }
 
+EAPI void
+e_dialog_parent_set(E_Dialog *dia, E_Win *win)
+{
+   if (!dia) return;
+   if (win)
+     ecore_x_icccm_transient_for_set(dia->win->evas_win, win->evas_win);
+   else
+     ecore_x_icccm_transient_for_unset(dia->win->evas_win);
+   ecore_evas_modal_set(dia->win->ecore_evas, !!win);
+}
+
 EAPI void
 e_dialog_show(E_Dialog *dia)
 {
    Evas_Coord mw, mh;
    Evas_Object *o;
-   
+
    o = dia->text_object;
    if (o)
      {
-	edje_object_size_min_calc(o, &mw, &mh);
-	edje_extern_object_min_size_set(o, mw, mh);
-	edje_object_part_swallow(dia->bg_object, "e.swallow.content", o);
+        edje_object_size_min_calc(o, &mw, &mh);
+        edje_extern_object_min_size_set(o, mw, mh);
+        edje_object_part_swallow(dia->bg_object, "e.swallow.content", o);
      }
 
    o = dia->box_object;
    e_widget_size_min_get(o, &mw, &mh);
    edje_extern_object_min_size_set(o, mw, mh);
    edje_object_part_swallow(dia->bg_object, "e.swallow.buttons", o);
-   
+
    edje_object_size_min_calc(dia->bg_object, &mw, &mh);
    e_win_resize(dia->win, mw, mh);
    e_win_size_min_set(dia->win, mw, mh);
@@ -246,7 +274,7 @@ e_dialog_show(E_Dialog *dia)
         e_util_win_auto_resize_fill(dia->win);
      }
    e_win_show(dia->win);
-   
+
    if (!e_widget_focus_get(dia->box_object))
      e_widget_focus_set(dia->box_object, 1);
 }
@@ -282,69 +310,71 @@ _e_dialog_cb_key_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED_
    dia = data;
    if (!strcmp(ev->keyname, "Tab"))
      {
-	if (evas_key_modifier_is_set(evas_key_modifier_get(e_win_evas_get(dia->win)), "Shift"))
-	  {
-	     if (e_widget_focus_get(dia->box_object))
-	       {
-		  if (!e_widget_focus_jump(dia->box_object, 0))
-		    {
-		       if (dia->text_object)
-			 e_widget_focus_set(dia->box_object, 0);
-		       else
-			 {
-			    e_widget_focus_set(dia->content_object, 0);
-			    if (!e_widget_focus_get(dia->content_object))
-			      e_widget_focus_set(dia->box_object, 0);
-			 }
-		    }
-	       }
-	     else
-	       {
-		  if (!e_widget_focus_jump(dia->content_object, 0))
-		    e_widget_focus_set(dia->box_object, 0);
-	       }
-	  }
-	else
-	  {
-	     if (e_widget_focus_get(dia->box_object))
-	       {
-		  if (!e_widget_focus_jump(dia->box_object, 1))
-		    {
-		       if (dia->text_object)
-			 e_widget_focus_set(dia->box_object, 1);
-		       else
-			 {
-			    e_widget_focus_set(dia->content_object, 1);
-			    if (!e_widget_focus_get(dia->content_object))
-			      e_widget_focus_set(dia->box_object, 1);
-			 }
-		    }
-	       }
-	     else
-	       {
-		  if (!e_widget_focus_jump(dia->content_object, 1))
-		    e_widget_focus_set(dia->box_object, 1);
-	       }
-	  }
+        if (evas_key_modifier_is_set(evas_key_modifier_get(e_win_evas_get(dia->win)), "Shift"))
+          {
+             if (e_widget_focus_get(dia->box_object))
+               {
+                  if (!e_widget_focus_jump(dia->box_object, 0))
+                    {
+                       if (dia->text_object)
+                         e_widget_focus_set(dia->box_object, 0);
+                       else
+                         {
+                            e_widget_focus_set(dia->content_object, 0);
+                            if (!e_widget_focus_get(dia->content_object))
+                              e_widget_focus_set(dia->box_object, 0);
+                         }
+                    }
+               }
+             else
+               {
+                  if (!e_widget_focus_jump(dia->content_object, 0))
+                    e_widget_focus_set(dia->box_object, 0);
+               }
+          }
+        else
+          {
+             if (e_widget_focus_get(dia->box_object))
+               {
+                  if (!e_widget_focus_jump(dia->box_object, 1))
+                    {
+                       if (dia->text_object)
+                         e_widget_focus_set(dia->box_object, 1);
+                       else
+                         {
+                            e_widget_focus_set(dia->content_object, 1);
+                            if (!e_widget_focus_get(dia->content_object))
+                              e_widget_focus_set(dia->box_object, 1);
+                         }
+                    }
+               }
+             else
+               {
+                  if (!e_widget_focus_jump(dia->content_object, 1))
+                    e_widget_focus_set(dia->box_object, 1);
+               }
+          }
      }
-   else if (((!strcmp(ev->keyname, "Return")) || 
-	     (!strcmp(ev->keyname, "KP_Enter"))))
+   else if (((!strcmp(ev->keyname, "Return")) ||
+             (!strcmp(ev->keyname, "KP_Enter"))))
      {
-	Evas_Object *o = NULL;
-	
-	if ((dia->content_object) && (e_widget_focus_get(dia->content_object)))
-	  o = e_widget_focused_object_get(dia->content_object);
-	else
-	  o = e_widget_focused_object_get(dia->box_object);
-	if (o) e_widget_activate(o);
+        Evas_Object *o = NULL;
+
+        if ((dia->content_object) && (e_widget_focus_get(dia->content_object)))
+          o = e_widget_focused_object_get(dia->content_object);
+        else
+          o = e_widget_focused_object_get(dia->box_object);
+        if (o) e_widget_activate(o);
      }
+   else if (!strcmp(ev->keyname, "Escape"))
+     e_object_del(E_OBJECT(dia));
 }
 
 static void
 _e_dialog_cb_delete(E_Win *win)
 {
    E_Dialog *dia;
-   
+
    dia = win->data;
    e_object_del(E_OBJECT(dia));
 }
@@ -353,7 +383,7 @@ static void
 _e_dialog_cb_resize(E_Win *win)
 {
    E_Dialog *dia;
-   
+
    dia = win->data;
    evas_object_resize(dia->bg_object, dia->win->w, dia->win->h);
 }
@@ -362,11 +392,11 @@ static void
 _e_dialog_cb_wid_on_focus(void *data, Evas_Object *obj)
 {
    E_Dialog *dia;
-   
+
    dia = data;
    if (obj == dia->content_object)
      e_widget_focused_object_clear(dia->box_object);
    else if (dia->content_object)
      e_widget_focused_object_clear(dia->content_object);
 }
-	  
+
