@@ -9,9 +9,7 @@ EAPI void
 e_error_message_show_internal(char *txt)
 {
    /* FIXME: maybe log these to a file and display them at some point */
-   printf("<<<< Enlightenment Error >>>>\n"
-	  "%s\n",
-	  txt);
+   printf("<<<< Enlightenment Error >>>>\n%s\n", txt);
 }
 
 /* local subsystem functions */
