@@ -2,14 +2,8 @@
 #include "e_fm_shared_codec.h"
 #include "e_fm_shared_device.h"
 
-#define TEBIBYTE_SIZE 1099511627776LL
-#define GIBIBYTE_SIZE 1073741824
-#define MEBIBYTE_SIZE 1048576
-#define KIBIBYTE_SIZE 1024
-
 static void _e_fm2_volume_write(E_Volume *v) EINA_ARG_NONNULL(1);
 static void _e_fm2_volume_erase(E_Volume *v) EINA_ARG_NONNULL(1);
-static void _e_fm2_device_mount_free(E_Fm2_Mount *m) EINA_ARG_NONNULL(1);
 static void _e_fm2_device_mount_ok(E_Fm2_Mount *m) EINA_ARG_NONNULL(1);
 static void _e_fm2_device_mount_fail(E_Fm2_Mount *m) EINA_ARG_NONNULL(1);
 static void _e_fm2_device_unmount_ok(E_Fm2_Mount *m) EINA_ARG_NONNULL(1);
@@ -18,28 +12,41 @@ static void _e_fm2_device_unmount_fail(E_Fm2_Mount *m) EINA_ARG_NONNULL(1);
 static Eina_List *_e_stores = NULL;
 static Eina_List *_e_vols = NULL;
 
+static Eina_Bool _check_run_show = EINA_FALSE;
+static Eina_Bool _check_run_hide = EINA_FALSE;
+static Ecore_Thread *_check_vols = NULL;
+
+static inline E_Fm2_Device_Mount_Op *
+_e_fm2_device_mount_op_new(char *args, size_t size, size_t length)
+{
+   E_Fm2_Device_Mount_Op *mop;
+
+   mop = E_NEW(E_Fm2_Device_Mount_Op, 1);
+   mop->args = args;
+   mop->size = size;
+   mop->length = length;
+   return mop;
+}
+
+static inline void
+_e_fm2_device_mount_op_free(E_Fm2_Device_Mount_Op *mop)
+{
+   if (!mop) return;
+   free(mop->args);
+   free(mop);
+}
+
 static void
 _e_fm2_device_volume_setup(E_Volume *v)
 {
    char label[1024] = {0};
-   char size[256] = {0};
+   char *size = NULL;
    const char *icon = NULL;
-   unsigned long long sz;
+   /* unsigned long long sz; */
 
    /* Compute the size in a readable form */
    if (v->size)
-     {
-        if ((sz = (v->size / TEBIBYTE_SIZE)) > 0)
-          snprintf(size, sizeof(size) - 1, _("%llu TiB"), sz);
-        else if ((sz = (v->size / GIBIBYTE_SIZE)) > 0)
-          snprintf(size, sizeof(size) - 1, _("%llu GiB"), sz);
-        else if ((sz = (v->size / MEBIBYTE_SIZE)) > 0)
-          snprintf(size, sizeof(size) - 1, _("%llu MiB"), sz);
-        else if ((sz = (v->size / KIBIBYTE_SIZE)) > 0)
-          snprintf(size, sizeof(size) - 1, _("%llu KiB"), sz);
-        else
-          snprintf(size, sizeof(size) - 1, _("%llu B"), v->size);
-     }
+     size = e_util_size_string_get(v->size);
 
    /* Choose the label */
    if ((v->label) && (v->label[0]))
@@ -49,27 +56,27 @@ _e_fm2_device_volume_setup(E_Volume *v)
    else if (((v->storage->vendor) && (v->storage->vendor[0])) &&
             ((v->storage->model) && (v->storage->model[0])))
      {
-        if (size[0] != '\0')
-          snprintf(label, sizeof(label) - 1, "%s %s - %s", v->storage->vendor, v->storage->model, size);
+        if (size && (size[0] != '\0'))
+          snprintf(label, sizeof(label) - 1, _("%s %s—%s"), v->storage->vendor, v->storage->model, size);
         else
           snprintf(label, sizeof(label) - 1, "%s %s", v->storage->vendor, v->storage->model);
      }
    else if ((v->storage->model) && (v->storage->model[0]))
      {
-        if (size[0] != '\0')
-          snprintf(label, sizeof(label) - 1, "%s - %s", v->storage->model, size);
+        if (size && (size[0] != '\0'))
+          snprintf(label, sizeof(label) - 1, _("%s—%s"), v->storage->model, size);
         else
           snprintf(label, sizeof(label) - 1, "%s", v->storage->model);
      }
    else if ((v->storage->vendor) && (v->storage->vendor[0]))
      {
-        if (size[0] != '\0')
-          snprintf(label, sizeof(label) - 1, "%s - %s", v->storage->vendor, size);
+        if (size && (size[0] != '\0'))
+          snprintf(label, sizeof(label) - 1, _("%s—%s"), v->storage->vendor, size);
         else
           snprintf(label, sizeof(label) - 1, "%s", v->storage->vendor);
      }
    else if (v->storage->drive_type && (!strcmp(v->storage->drive_type, "sd_mmc")))
-     snprintf(label, sizeof(label) - 1, "Flash Card - %s", size);
+     snprintf(label, sizeof(label) - 1, _("Flash Card—%s"), size);
    else
      snprintf(label, sizeof(label), _("Unknown Volume"));
 
@@ -104,18 +111,21 @@ _e_fm2_device_volume_setup(E_Volume *v)
    if (icon) eina_stringshare_replace(&v->icon, icon);
 
    if ((!v->mount_point) ||
+       // filter out these mountouts - hack hack
        (strcmp(v->mount_point, "/") &&
         strcmp(v->mount_point, "/home") &&
         strcmp(v->mount_point, "/tmp")))
      _e_fm2_volume_write(v);
+
+   E_FREE(size);
 }
-     
+
 EAPI void
 e_fm2_device_storage_add(E_Storage *s)
 {
    Eina_List *l;
    E_Volume *v;
-   
+
    if (e_fm2_device_storage_find(s->udi)) return;
 
    s->validated = EINA_TRUE;
@@ -165,7 +175,7 @@ e_fm2_device_storage_add(E_Storage *s)
      {
         s->trackable = EINA_TRUE;
      }
-     
+
    EINA_LIST_FOREACH(_e_vols, l, v) /* catch volumes which were added before their storage */
      {
         if ((!v->storage) && (s->udi == v->parent))
@@ -238,7 +248,7 @@ e_fm2_device_volume_add(E_Volume *v)
    if ((v->efm_mode == EFM_MODE_USING_HAL_MOUNT) &&
        ((!v->mount_point) || (!v->mount_point[0])))
      {
-        if (v->mount_point) eina_stringshare_del(v->mount_point);
+        eina_stringshare_del(v->mount_point);
         v->mount_point = NULL;
         v->mount_point = e_fm2_device_volume_mountpoint_get(v);
         if ((!v->mount_point) || (!v->mount_point[0]))
@@ -270,7 +280,7 @@ e_fm2_device_volume_add(E_Volume *v)
      }
 
    if (v->storage) _e_fm2_device_volume_setup(v);
-     
+
 }
 
 EAPI void
@@ -289,7 +299,15 @@ e_fm2_device_volume_del(E_Volume *v)
    EINA_LIST_FREE(v->mounts, m)
      {
         _e_fm2_device_unmount_ok(m);
-        _e_fm2_device_mount_free(m);
+        e_fm2_device_mount_free(m);
+     }
+   while (v->mount_ops)
+     {
+        E_Fm2_Device_Mount_Op *mop;
+
+        mop = (E_Fm2_Device_Mount_Op*)v->mount_ops;
+        v->mount_ops = eina_inlist_remove(v->mount_ops, v->mount_ops);
+        _e_fm2_device_mount_op_free(mop);
      }
    _e_fm_shared_device_volume_free(v);
 }
@@ -303,7 +321,6 @@ _e_fm2_volume_write(E_Volume *v)
 
    if (!v->storage) return;
    id = ecore_file_file_get(v->storage->udi);
-//   printf("vol write %s\n", id);
    e_user_dir_snprintf(buf, sizeof(buf), "fileman/favorites/|%s_%d.desktop",
                        id, v->partition_number);
 
@@ -329,23 +346,17 @@ _e_fm2_volume_write(E_Volume *v)
 
         if (e_config->device_desktop)
           {
-             e_user_homedir_snprintf(buf2, sizeof(buf2),
-                                     "%s/|%s_%d.desktop",
-                                     _("Desktop"), id, v->partition_number);
+             snprintf(buf2, sizeof(buf2), "%s/|%s_%d.desktop",
+                      efreet_desktop_dir_get(), id, v->partition_number);
              ecore_file_symlink(buf, buf2);
           }
 
         /* FIXME: manipulate icon directly */
         _e_fm2_file_force_update(buf);
-        //_e_fm2_file_force_update(buf2);
+        _e_fm2_file_force_update(buf2);
      }
 }
 
-#undef TEBIBYTE_SIZE
-#undef GIBIBYTE_SIZE
-#undef MEBIBYTE_SIZE
-#undef KIBIBYTE_SIZE
-
 static void
 _e_fm2_volume_erase(E_Volume *v)
 {
@@ -354,19 +365,18 @@ _e_fm2_volume_erase(E_Volume *v)
 
    if (!v->storage) return;
    id = ecore_file_file_get(v->storage->udi);
-   e_user_homedir_snprintf(buf, sizeof(buf), "%s/|%s_%d.desktop",
-                           _("Desktop"), id, v->partition_number);
-   ecore_file_unlink(buf);
-   _e_fm2_file_force_update(buf);
-
    if (e_config->device_desktop)
      {
-        e_user_dir_snprintf(buf, sizeof(buf),
-                            "fileman/favorites/|%s_%d.desktop",
-                            id, v->partition_number);
+        snprintf(buf, sizeof(buf), "%s/|%s_%d.desktop",
+                 efreet_desktop_dir_get(), id, v->partition_number);
         ecore_file_unlink(buf);
         _e_fm2_file_force_update(buf);
      }
+   e_user_dir_snprintf(buf, sizeof(buf),
+                       "fileman/favorites/|%s_%d.desktop",
+                       id, v->partition_number);
+   ecore_file_unlink(buf);
+   _e_fm2_file_force_update(buf);
 }
 
 EAPI E_Volume *
@@ -386,6 +396,20 @@ e_fm2_device_volume_find(const char *udi)
    return NULL;
 }
 
+EAPI E_Volume *
+e_fm2_device_volume_find_fast(const char *udi)
+{
+   Eina_List *l;
+   E_Volume *v;
+
+   if (!udi) return NULL;
+
+   EINA_LIST_FOREACH(_e_vols, l, v)
+     if (udi == v->udi) return v;
+
+   return NULL;
+}
+
 EAPI const char *
 e_fm2_device_volume_mountpoint_get(E_Volume *v)
 {
@@ -415,20 +439,25 @@ e_fm2_device_volume_mountpoint_get(E_Volume *v)
    return eina_stringshare_add(buf);
 }
 
+EAPI E_Fm2_Device_Mount_Op *
+e_fm2_device_mount_op_add(E_Fm2_Mount *m, char *args, size_t size, size_t length)
+{
+   E_Fm2_Device_Mount_Op *mop;
+
+   mop = _e_fm2_device_mount_op_new(args, size, length);
+   m->volume->mount_ops = eina_inlist_append(m->volume->mount_ops, EINA_INLIST_GET(mop));
+   return mop;
+}
+
 EAPI void
 e_fm2_device_mount_add(E_Volume   *v,
                        const char *mountpoint)
 {
-   Eina_List *l;
-   E_Fm2_Mount *m;
-
    v->mounted = EINA_TRUE;
    if (mountpoint && (mountpoint[0]))
      eina_stringshare_replace(&v->mount_point, mountpoint);
 
-   EINA_LIST_FOREACH(v->mounts, l, m)
-     _e_fm2_device_mount_ok(m);
-
+   E_LIST_FOREACH(v->mounts, _e_fm2_device_mount_ok);
 //   printf("MOUNT %s %s\n", v->udi, v->mount_point);
 }
 
@@ -447,17 +476,17 @@ e_fm2_device_mount_del(E_Volume *v)
    EINA_LIST_FREE(v->mounts, m)
      {
         _e_fm2_device_unmount_ok(m);
-        _e_fm2_device_mount_free(m);
+        e_fm2_device_mount_free(m);
      }
 }
 
 EAPI void
-_e_fm2_device_mount_free(E_Fm2_Mount *m)
+e_fm2_device_mount_free(E_Fm2_Mount *m)
 {
    if (!m) return;
 
-   if (m->udi) eina_stringshare_del(m->udi);
-   if (m->mount_point) eina_stringshare_del(m->mount_point);
+   eina_stringshare_del(m->udi);
+   eina_stringshare_del(m->mount_point);
 
    free(m);
 }
@@ -468,6 +497,8 @@ e_fm2_device_mount_find(const char *path)
    Eina_List *l;
    E_Volume *v;
 
+   if (!path) return NULL;
+
    EINA_LIST_FOREACH(_e_vols, l, v)
      {
         if (v->mounted
@@ -504,10 +535,10 @@ e_fm2_device_mount(E_Volume *v,
 
    v->mounts = eina_list_prepend(v->mounts, m);
 
-//   printf("BEGIN MOUNT %p '%s'\n", m, v->mount_point);
 
    if (!v->mounted)
      {
+        //printf("BEGIN MOUNT %p '%s'\n", m, v->mount_point);
         v->auto_unmount = EINA_TRUE;
         _e_fm2_client_mount(v->udi, v->mount_point);
      }
@@ -535,7 +566,7 @@ e_fm2_device_mount_fail(E_Volume *v)
    EINA_LIST_FREE(v->mounts, m)
      {
         _e_fm2_device_mount_fail(m);
-        _e_fm2_device_mount_free(m);
+        e_fm2_device_mount_free(m);
      }
 }
 
@@ -545,11 +576,19 @@ e_fm2_device_unmount(E_Fm2_Mount *m)
    E_Volume *v;
 
    if (!(v = m->volume)) return;
+   if (v->mount_ops)
+     {
+        m->deleted = EINA_TRUE;
+        return;
+     }
    v->mounts = eina_list_remove(v->mounts, m);
-   _e_fm2_device_mount_free(m);
+   e_fm2_device_mount_free(m);
 
    if (v->auto_unmount && v->mounted && !eina_list_count(v->mounts))
-     _e_fm2_client_unmount(v->udi);
+     {
+        //printf("BEGIN UNMOUNT %p '%s'\n", m, v->udi);
+        _e_fm2_client_unmount(v->udi);
+     }
 }
 
 EAPI void
@@ -573,12 +612,14 @@ _e_fm2_device_mount_ok(E_Fm2_Mount *m)
      m->mount_point = eina_stringshare_add(m->volume->mount_point);
    if (m->mount_ok)
      m->mount_ok(m->data);
-//   printf("MOUNT OK '%s'\n", m->mount_point);
+   if (m->deleted) ecore_job_add((Ecore_Cb)e_fm2_device_unmount, m);
+   //printf("MOUNT OK '%s'\n", m->mount_point);
 }
 
 static void
 _e_fm2_device_mount_fail(E_Fm2_Mount *m)
 {
+   //printf("MOUNT FAIL '%s'\n", m->mount_point);
    m->mounted = EINA_FALSE;
    if (m->mount_point)
      {
@@ -612,6 +653,45 @@ _e_fm2_device_unmount_fail(E_Fm2_Mount *m)
      m->unmount_fail(m->data);
 }
 
+static void
+_e_fm2_device_check_desktop_icons_cb_end(void *data __UNUSED__, Ecore_Thread *eth __UNUSED__)
+{
+   _check_vols = NULL;
+   if (_check_run_show)
+     e_fm2_device_show_desktop_icons();
+   else if (_check_run_hide)
+     e_fm2_device_hide_desktop_icons();
+   _check_run_hide = _check_run_show = EINA_FALSE;
+}
+
+static void
+_e_fm2_device_check_desktop_icons_list_cb(const char *name, const char *path, void *data)
+{
+   char buf[PATH_MAX], buf2[PATH_MAX];
+   Eina_List *l;
+   char *file;
+
+   if (path[0] != '|') return; // not e17 link
+   if (!eina_str_has_extension(name, ".desktop")) return;
+   snprintf(buf, sizeof(buf), "%s/%s", path, name);
+   if (ecore_file_size(buf) > 200) return; // unlikely that we're going to write a desktop file this long
+   e_user_dir_snprintf(buf2, sizeof(buf2), "fileman/favorites/%s", name);
+   if (!ecore_file_exists(buf2)) return;
+   EINA_LIST_FOREACH(data, l, file)
+     if (!strcmp(file, buf)) return; // valid link
+
+   ecore_file_unlink(buf);
+   ecore_file_unlink(buf2);
+}
+
+static void
+_e_fm2_device_check_desktop_icons_cb(void *data, Ecore_Thread *eth __UNUSED__)
+{
+   eina_file_dir_list(efreet_desktop_dir_get(), EINA_FALSE, _e_fm2_device_check_desktop_icons_list_cb, data);
+
+   E_FREE_LIST(data, free);
+}
+
 EAPI void
 e_fm2_device_show_desktop_icons(void)
 {
@@ -621,6 +701,13 @@ e_fm2_device_show_desktop_icons(void)
    char buf2[PATH_MAX] = {0};
    const char *id;
 
+   if (_check_vols)
+     {
+        _check_run_hide = EINA_FALSE;
+        _check_run_show = EINA_TRUE;
+        return;
+     }
+
    EINA_LIST_FOREACH(_e_vols, l, v)
      {
         if (!v) continue;
@@ -632,9 +719,8 @@ e_fm2_device_show_desktop_icons(void)
                             "fileman/favorites/|%s_%d.desktop",
                             id, v->partition_number);
 
-        e_user_homedir_snprintf(buf2, sizeof(buf2),
-                                "%s/|%s_%d.desktop",
-                                _("Desktop"), id, v->partition_number);
+        snprintf(buf2, sizeof(buf2), "%s/|%s_%d.desktop",
+                 efreet_desktop_dir_get(), id, v->partition_number);
 
         if (ecore_file_exists(buf) && !ecore_file_exists(buf2))
           {
@@ -652,6 +738,13 @@ e_fm2_device_hide_desktop_icons(void)
    char buf[PATH_MAX] = {0};
    const char *id;
 
+   if (_check_vols)
+     {
+        _check_run_show = EINA_FALSE;
+        _check_run_hide = EINA_TRUE;
+        return;
+     }
+
    EINA_LIST_FOREACH(_e_vols, l, v)
      {
         if (!v) continue;
@@ -659,9 +752,8 @@ e_fm2_device_hide_desktop_icons(void)
 
         id = ecore_file_file_get(v->storage->udi);
 
-        e_user_homedir_snprintf(buf, sizeof(buf),
-                                "%s/|%s_%d.desktop",
-                                _("Desktop"), id, v->partition_number);
+        snprintf(buf, sizeof(buf), "%s/|%s_%d.desktop",
+                 efreet_desktop_dir_get(), id, v->partition_number);
 
         if (ecore_file_exists(buf))
           {
@@ -671,9 +763,31 @@ e_fm2_device_hide_desktop_icons(void)
      }
 }
 
+EAPI void
+e_fm2_device_check_desktop_icons(void)
+{
+   Eina_List *l, *thd = NULL;
+   E_Volume *v;
+   char buf[PATH_MAX] = {0};
+   const char *id;
+
+   if (_check_vols) return;
+   EINA_LIST_FOREACH(_e_vols, l, v)
+     {
+        if (!v) continue;
+        if (!v->storage) continue;
+
+        id = ecore_file_file_get(v->storage->udi);
+
+        snprintf(buf, sizeof(buf), "%s/|%s_%d.desktop",
+                 efreet_desktop_dir_get(), id, v->partition_number);
+        thd = eina_list_append(thd, strdup(buf));
+     }
+   _check_vols = ecore_thread_run(_e_fm2_device_check_desktop_icons_cb, _e_fm2_device_check_desktop_icons_cb_end, _e_fm2_device_check_desktop_icons_cb_end, thd);
+}
+
 EAPI Eina_List *
 e_fm2_device_volume_list_get(void)
 {
    return _e_vols;
 }
-
