@@ -1,10 +1,10 @@
 #include "e.h"
 
 /* PROTOTYPES - same all the time */
-static void *_create_data(E_Config_Dialog *cfd);
-static void _free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
-static int _basic_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
-static int _advanced_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
+static void        *_create_data(E_Config_Dialog *cfd);
+static void         _free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
+static int          _basic_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
+static int          _advanced_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
 static Evas_Object *_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
 static Evas_Object *_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
 
@@ -13,30 +13,32 @@ struct _E_Config_Dialog_Data
 {
    E_Border *border;
    /*- BASIC -*/
-   int do_what_i_say;
-   int protect_from_me;
-   int keep_my_border;
-   int important_window;
-   int remember_locks;
+   int       do_what_i_say;
+   int       protect_from_me;
+   int       keep_my_border;
+   int       important_window;
+   int       remember_locks;
    /*- ADVANCED -*/
-   struct {
-      struct {
-	 int location;
-	 int size;
-	 int stacking;
-	 int iconify;
-	 int desk; /* unused atm */
-	 int sticky;
-	 int shade;
-	 int maximize;
-	 int fullscreen;
+   struct
+   {
+      struct
+      {
+         int location;
+         int size;
+         int stacking;
+         int iconify;
+         int desk; /* unused atm */
+         int sticky;
+         int shade;
+         int maximize;
+         int fullscreen;
       } user, client;
       int border;
       int close;
       int focus_in; /* unused atm */
       int focus_out; /* unused atm */
       int life;
-      
+
       int remember;
    } lock;
 };
@@ -47,25 +49,25 @@ e_int_border_locks(E_Border *bd)
 {
    E_Config_Dialog *cfd;
    E_Config_Dialog_View *v;
-   
+
    v = E_NEW(E_Config_Dialog_View, 1);
    if (v)
-      {
-         /* methods */
-         v->create_cfdata           = _create_data;
-         v->free_cfdata             = _free_data;
-         v->basic.apply_cfdata      = _basic_apply_data;
-         v->basic.create_widgets    = _basic_create_widgets;
-         v->advanced.apply_cfdata   = _advanced_apply_data;
-         v->advanced.create_widgets = _advanced_create_widgets;
-	 v->override_auto_apply = 1;
-         /* create config diaolg for bd object/data */
-         cfd = e_config_dialog_new(bd->zone->container,
-				   _("Window Locks"),
-				   "E", "_border_locks_dialog",
-				   NULL, 0, v, bd);
-         bd->border_locks_dialog = cfd;
-      }
+     {
+        /* methods */
+        v->create_cfdata = _create_data;
+        v->free_cfdata = _free_data;
+        v->basic.apply_cfdata = _basic_apply_data;
+        v->basic.create_widgets = _basic_create_widgets;
+        v->advanced.apply_cfdata = _advanced_apply_data;
+        v->advanced.create_widgets = _advanced_create_widgets;
+        v->override_auto_apply = 1;
+        /* create config diaolg for bd object/data */
+        cfd = e_config_dialog_new(bd->zone->container,
+                                  _("Window Locks"),
+                                  "E", "_border_locks_dialog",
+                                  NULL, 0, v, bd);
+        bd->border_locks_dialog = cfd;
+     }
 }
 
 /**--CREATE--**/
@@ -98,7 +100,7 @@ _fill_data(E_Config_Dialog_Data *cfdata)
    if ((cfdata->border->remember) &&
        (cfdata->border->remember->apply & E_REMEMBER_APPLY_LOCKS))
      cfdata->lock.remember = 1;
-     
+
    /* figure out basic config from the current locks */
    if (cfdata->lock.user.location || cfdata->lock.user.size ||
        cfdata->lock.user.stacking || cfdata->lock.user.iconify ||
@@ -127,7 +129,7 @@ _create_data(E_Config_Dialog *cfd)
     * the running systems/config in the apply methods
     */
    E_Config_Dialog_Data *cfdata;
-   
+
    cfdata = E_NEW(E_Config_Dialog_Data, 1);
    cfdata->border = cfd->data;
    _fill_data(cfdata);
@@ -148,7 +150,7 @@ _basic_apply_data(E_Config_Dialog *cfd __UNUSED__, E_Config_Dialog_Data *cfdata)
 {
    int flag;
    /* Actually take our cfdata settings and apply them in real life */
-   
+
    flag = cfdata->do_what_i_say;
    cfdata->border->lock_client_location = flag;
    cfdata->border->lock_client_size = flag;
@@ -174,37 +176,37 @@ _basic_apply_data(E_Config_Dialog *cfd __UNUSED__, E_Config_Dialog_Data *cfdata)
    flag = cfdata->important_window;
    cfdata->border->lock_close = flag;
    cfdata->border->lock_life = flag;
-   
+
    flag = cfdata->keep_my_border;
    cfdata->border->lock_border = flag;
-   
+
    if (cfdata->remember_locks)
      {
-	if (!cfdata->border->remember)
-	  {
-	     cfdata->border->remember = e_remember_new();
-	     if (cfdata->border->remember)
-		  e_remember_use(cfdata->border->remember);
-	  }
-	if (cfdata->border->remember)
-	  {
-	     cfdata->border->remember->apply |= E_REMEMBER_APPLY_LOCKS;
+        if (!cfdata->border->remember)
+          {
+             cfdata->border->remember = e_remember_new();
+             if (cfdata->border->remember)
+               e_remember_use(cfdata->border->remember);
+          }
+        if (cfdata->border->remember)
+          {
+             cfdata->border->remember->apply |= E_REMEMBER_APPLY_LOCKS;
              e_remember_default_match_set(cfdata->border->remember, cfdata->border);
-	     e_remember_update(cfdata->border);
-	  }
+             e_remember_update(cfdata->border);
+          }
      }
    else
      {
-	if (cfdata->border->remember)
-	  {
-	     cfdata->border->remember->apply &= ~E_REMEMBER_APPLY_LOCKS;
-	     if (cfdata->border->remember->apply == 0)
-	       {
-		  e_remember_unuse(cfdata->border->remember);
-		  e_remember_del(cfdata->border->remember);
-		  cfdata->border->remember = NULL;
-	       }
-	  }
+        if (cfdata->border->remember)
+          {
+             cfdata->border->remember->apply &= ~E_REMEMBER_APPLY_LOCKS;
+             if (cfdata->border->remember->apply == 0)
+               {
+                  e_remember_unuse(cfdata->border->remember);
+                  e_remember_del(cfdata->border->remember);
+                  cfdata->border->remember = NULL;
+               }
+          }
      }
    e_config_save_queue();
    return 1; /* Apply was OK */
@@ -240,31 +242,31 @@ _advanced_apply_data(E_Config_Dialog *cfd __UNUSED__, E_Config_Dialog_Data *cfda
 
    if (cfdata->lock.remember)
      {
-	if (!cfdata->border->remember)
-	  {
-	     cfdata->border->remember = e_remember_new();
-	     if (cfdata->border->remember)
-		  e_remember_use(cfdata->border->remember);
-	  }
-	if (cfdata->border->remember)
-	  {
-	     cfdata->border->remember->apply |= E_REMEMBER_APPLY_LOCKS;
+        if (!cfdata->border->remember)
+          {
+             cfdata->border->remember = e_remember_new();
+             if (cfdata->border->remember)
+               e_remember_use(cfdata->border->remember);
+          }
+        if (cfdata->border->remember)
+          {
+             cfdata->border->remember->apply |= E_REMEMBER_APPLY_LOCKS;
              e_remember_default_match_set(cfdata->border->remember, cfdata->border);
-	     e_remember_update(cfdata->border);
-	  }
+             e_remember_update(cfdata->border);
+          }
      }
    else
      {
-	if (cfdata->border->remember)
-	  {
-	     cfdata->border->remember->apply &= ~E_REMEMBER_APPLY_LOCKS;
-	     if (cfdata->border->remember->apply == 0)
-	       {
-		  e_remember_unuse(cfdata->border->remember);
-		  e_remember_del(cfdata->border->remember);
-		  cfdata->border->remember = NULL;
-	       }
-	  }
+        if (cfdata->border->remember)
+          {
+             cfdata->border->remember->apply &= ~E_REMEMBER_APPLY_LOCKS;
+             if (cfdata->border->remember->apply == 0)
+               {
+                  e_remember_unuse(cfdata->border->remember);
+                  e_remember_del(cfdata->border->remember);
+                  cfdata->border->remember = NULL;
+               }
+          }
      }
    e_config_save_queue();
    return 1; /* Apply was OK */
@@ -276,20 +278,20 @@ _basic_create_widgets(E_Config_Dialog *cfd __UNUSED__, Evas *evas, E_Config_Dial
 {
    /* generate the core widget layout for a basic dialog */
    Evas_Object *o, *ob, *of;
-   
+
    o = e_widget_list_add(evas, 0, 0);
-   
+
    of = e_widget_framelist_add(evas, _("Generic Locks"), 0);
-   ob = e_widget_check_add(evas, _("Lock the Window so it does only what I tell it to"), &(cfdata->do_what_i_say));
+   ob = e_widget_check_add(evas, _("Prevent this window from moving on its own"), &(cfdata->do_what_i_say));
    e_widget_framelist_object_append(of, ob);
-   ob = e_widget_check_add(evas, _("Protect this window from me accidentally changing it"), &(cfdata->protect_from_me));
+   ob = e_widget_check_add(evas, _("Prevent this window from being changed by me"), &(cfdata->protect_from_me));
    e_widget_framelist_object_append(of, ob);
-   ob = e_widget_check_add(evas, _("Protect this window from being accidentally closed because it is important"), &(cfdata->important_window));
+   ob = e_widget_check_add(evas, _("Prevent this window from being closed"), &(cfdata->important_window));
    e_widget_framelist_object_append(of, ob);
    ob = e_widget_check_add(evas, _("Do not allow the border to change on this window"), &(cfdata->keep_my_border));
    e_widget_framelist_object_append(of, ob);
    e_widget_list_object_append(o, of, 1, 1, 0.5);
-   ob = e_widget_check_add(evas, _("Remember these Locks for this window next time it appears"), &(cfdata->remember_locks));
+   ob = e_widget_check_add(evas, _("Remember the locks for this window"), &(cfdata->remember_locks));
    e_widget_list_object_append(o, ob, 0, 0, 0.0);
    return o;
 }
@@ -298,11 +300,12 @@ static Evas_Object *
 _advanced_create_widgets(E_Config_Dialog *cfd __UNUSED__, Evas *evas, E_Config_Dialog_Data *cfdata)
 {
    /* generate the core widget layout for an advanced dialog */
-   Evas_Object *o, *ob, *of;
-   
-   o = e_widget_list_add(evas, 0, 0);
-   
-   of = e_widget_frametable_add(evas, _("Lock program changing:"), 0);
+   Evas_Object *o, *ob, *of, *ol;
+
+   ol = e_widget_list_add(evas, 0, 0);
+   o = e_widget_toolbook_add(evas, (48 * e_scale), (48 * e_scale));
+
+   of = e_widget_frametable_add(evas, _("Prevent Changes In:"), 0);
    ob = e_widget_check_add(evas, _("Position"), &(cfdata->lock.client.location));
    e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 1, 1, 1);
    ob = e_widget_check_add(evas, _("Size"), &(cfdata->lock.client.size));
@@ -319,9 +322,9 @@ _advanced_create_widgets(E_Config_Dialog *cfd __UNUSED__, Evas *evas, E_Config_D
    e_widget_frametable_object_append(of, ob, 1, 2, 1, 1, 1, 1, 1, 1);
    ob = e_widget_check_add(evas, _("Fullscreen state"), &(cfdata->lock.client.fullscreen));
    e_widget_frametable_object_append(of, ob, 1, 3, 1, 1, 1, 1, 1, 1);
-   e_widget_list_object_append(o, of, 1, 1, 0.5);
+   e_widget_toolbook_page_append(o, NULL, _("Program Locks"), of, 1, 1, 1, 1, 0.5, 0.0);
 
-   of = e_widget_frametable_add(evas, _("Lock me from changing:"), 0);
+   of = e_widget_frametable_add(evas, _("Prevent Changes In:"), 0);
    ob = e_widget_check_add(evas, _("Position"), &(cfdata->lock.user.location));
    e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 1, 1, 1);
    ob = e_widget_check_add(evas, _("Size"), &(cfdata->lock.user.size));
@@ -340,17 +343,21 @@ _advanced_create_widgets(E_Config_Dialog *cfd __UNUSED__, Evas *evas, E_Config_D
    e_widget_frametable_object_append(of, ob, 1, 2, 1, 1, 1, 1, 1, 1);
    ob = e_widget_check_add(evas, _("Border style"), &(cfdata->lock.border));
    e_widget_frametable_object_append(of, ob, 1, 3, 1, 1, 1, 1, 1, 1);
-   e_widget_list_object_append(o, of, 1, 1, 0.5);
-   
-   of = e_widget_framelist_add(evas, _("Stop me from:"), 0);
+   e_widget_toolbook_page_append(o, NULL, _("User Locks"), of, 1, 1, 1, 1, 0.5, 0.0);
+
+   of = e_widget_framelist_add(evas, _("Prevent:"), 0);
    ob = e_widget_check_add(evas, _("Closing the window"), &(cfdata->lock.close));
    e_widget_framelist_object_append(of, ob);
-   ob = e_widget_check_add(evas, _("Exiting my login with this window open"), &(cfdata->lock.life));
+   ob = e_widget_check_add(evas, _("Logging out while this window is open"), &(cfdata->lock.life));
    e_widget_framelist_object_append(of, ob);
-   e_widget_list_object_append(o, of, 1, 1, 0.5);
-   
+   e_widget_toolbook_page_append(o, NULL, _("Behavior Locks"), of, 1, 1, 1, 1, 0.5, 0.0);
+   e_widget_toolbook_page_show(o, 0);
+   e_widget_list_object_append(ol, o, 1, 1, 0.5);
+
    ob = e_widget_check_add(evas, _("Remember these Locks"), &(cfdata->lock.remember));
-   e_widget_list_object_append(o, ob, 1, 1, 0.5);
+   e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    
-   return o;
+
+   return ol;
 }
+
