@@ -1,21 +1,21 @@
 #include "e.h"
 
-#define SMART_NAME "e_livethumb"
-#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
-#define INTERNAL_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if (!sd) return;
+#define SMART_NAME     "e_livethumb"
+#define API_ENTRY      E_Smart_Data * sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
+#define INTERNAL_ENTRY E_Smart_Data * sd; sd = evas_object_smart_data_get(obj); if (!sd) return;
 typedef struct _E_Smart_Data E_Smart_Data;
 typedef struct _E_Smart_Item E_Smart_Item;
 
 struct _E_Smart_Data
-{ 
+{
    Evas_Coord   x, y, w, h;
-   
-   Evas_Object   *smart_obj;
-   Evas_Object   *evas_obj;
-   Evas_Object   *thumb_obj;
-   Evas          *evas;
-   Evas_Coord     vw, vh;
-}; 
+
+   Evas_Object *smart_obj;
+   Evas_Object *evas_obj;
+   Evas_Object *thumb_obj;
+   Evas        *evas;
+   Evas_Coord   vw, vh;
+};
 
 /* local subsystem functions */
 static void _e_smart_reconfigure(E_Smart_Data *sd);
@@ -26,7 +26,7 @@ static void _e_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h);
 static void _e_smart_show(Evas_Object *obj);
 static void _e_smart_hide(Evas_Object *obj);
 static void _e_smart_color_set(Evas_Object *obj, int r, int g, int b, int a);
-static void _e_smart_clip_set(Evas_Object *obj, Evas_Object * clip);
+static void _e_smart_clip_set(Evas_Object *obj, Evas_Object *clip);
 static void _e_smart_clip_unset(Evas_Object *obj);
 static void _e_smart_init(void);
 
@@ -67,17 +67,35 @@ e_livethumb_vsize_get(Evas_Object *obj, Evas_Coord *w, Evas_Coord *h)
    if (h) *h = sd->vh;
 }
 
+static void
+_e_livethumb_edje_preloaded(void *data EINA_UNUSED, Evas_Object *obj,
+                            const char *emission EINA_UNUSED,
+                            const char *source EINA_UNUSED)
+{
+   evas_object_show(obj);
+}
+
 EAPI void
 e_livethumb_thumb_set(Evas_Object *obj, Evas_Object *thumb)
 {
    API_ENTRY return;
    if (!thumb)
      {
-	sd->thumb_obj = NULL;
-	return;
+        sd->thumb_obj = NULL;
+        return;
      }
    sd->thumb_obj = thumb;
-   evas_object_show(sd->thumb_obj);
+   if (!strcmp(evas_object_type_get(thumb), "edje"))
+     {
+        edje_object_signal_callback_add(thumb,
+                                        "preload,done", "",
+                                        _e_livethumb_edje_preloaded, NULL);
+        edje_object_preload(thumb, EINA_FALSE);
+     }
+   else
+     {
+        evas_object_show(sd->thumb_obj);
+     }
    evas_object_move(sd->thumb_obj, 0, 0);
    evas_object_resize(sd->thumb_obj, sd->vw, sd->vh);
 }
@@ -88,7 +106,7 @@ e_livethumb_thumb_get(Evas_Object *obj)
    API_ENTRY return NULL;
    return sd->thumb_obj;
 }
-   
+
 /* local subsystem functions */
 
 static void
@@ -103,11 +121,11 @@ static void
 _e_smart_add(Evas_Object *obj)
 {
    E_Smart_Data *sd;
-   
+
    sd = calloc(1, sizeof(E_Smart_Data));
    if (!sd) return;
    evas_object_smart_data_set(obj, sd);
-   
+
    sd->smart_obj = obj;
    sd->x = 0;
    sd->y = 0;
@@ -175,7 +193,7 @@ _e_smart_color_set(Evas_Object *obj, int r, int g, int b, int a)
 }
 
 static void
-_e_smart_clip_set(Evas_Object *obj, Evas_Object * clip)
+_e_smart_clip_set(Evas_Object *obj, Evas_Object *clip)
 {
    INTERNAL_ENTRY;
    evas_object_clip_set(sd->evas_obj, clip);
@@ -186,7 +204,7 @@ _e_smart_clip_unset(Evas_Object *obj)
 {
    INTERNAL_ENTRY;
    evas_object_clip_unset(sd->evas_obj);
-}  
+}
 
 /* never need to touch this */
 
@@ -194,29 +212,29 @@ static void
 _e_smart_init(void)
 {
    if (_e_smart) return;
-     {
-	static const Evas_Smart_Class sc =
-	  {
-	     SMART_NAME,
-	       EVAS_SMART_CLASS_VERSION,
-	       _e_smart_add,
-	       _e_smart_del, 
-	       _e_smart_move,
-	       _e_smart_resize,
-	       _e_smart_show,
-	       _e_smart_hide,
-	       _e_smart_color_set,
-	       _e_smart_clip_set,
-	       _e_smart_clip_unset,
-	       NULL,
-	       NULL,
-	       NULL,
-	       NULL,
-	       NULL,
-	       NULL,
-	       NULL
-	  };
-	_e_smart = evas_smart_class_new(&sc);
-     }
+   {
+      static const Evas_Smart_Class sc =
+      {
+         SMART_NAME,
+         EVAS_SMART_CLASS_VERSION,
+         _e_smart_add,
+         _e_smart_del,
+         _e_smart_move,
+         _e_smart_resize,
+         _e_smart_show,
+         _e_smart_hide,
+         _e_smart_color_set,
+         _e_smart_clip_set,
+         _e_smart_clip_unset,
+         NULL,
+         NULL,
+         NULL,
+         NULL,
+         NULL,
+         NULL,
+         NULL
+      };
+      _e_smart = evas_smart_class_new(&sc);
+   }
 }
 
