@@ -18,10 +18,12 @@ struct _E_Smart_Data
    double         val, val_min, val_max, val_range, step_size;
    int            reversed, step_count, horizontal;
    int            direction;
+   int            changing;
    const char    *format;
    Evas_Coord     minw, minh;
    Ecore_Timer   *set_timer;
    Eina_List     *special_values;
+   Eina_Bool      disabled : 1;
 };
 
 struct _E_Slider_Special_Value
@@ -204,6 +206,13 @@ e_slider_direction_set(Evas_Object *obj, int reversed)
    _e_smart_value_update_now(sd);
 }
 
+EAPI void
+e_slider_disabled_set(Evas_Object *obj, Eina_Bool disable)
+{
+   API_ENTRY return;
+   sd->disabled = disable;
+}
+
 EAPI int
 e_slider_direction_get(Evas_Object *obj)
 {
@@ -253,8 +262,12 @@ _e_smart_set_timer(void *data)
    if (pos < 0.0) pos = 0.0;
    else if (pos > 1.0) pos = 1.0;
    if (sd->reversed) pos = 1.0 - pos;
-   edje_object_part_drag_value_set(sd->edje_obj, "e.dragable.slider", pos, pos);
+   sd->changing++;
+   if (sd->set_timer) ecore_timer_del(sd->set_timer);
    sd->set_timer = NULL;
+   edje_object_part_drag_value_set(sd->edje_obj, "e.dragable.slider", pos, pos);
+   edje_object_message_signal_process(sd->edje_obj);
+   sd->changing--;
    return ECORE_CALLBACK_CANCEL;
 }
 
@@ -353,6 +366,7 @@ _e_smart_signal_cb_drag(void *data, Evas_Object *obj __UNUSED__, const char *emi
    _e_smart_value_fetch(sd);
    _e_smart_value_limit(sd);
    _e_smart_format_update(sd);
+   if (sd->changing) return;
    if (sd->val != pval)
      evas_object_smart_callback_call(sd->smart_obj, "changed", NULL);
 }
@@ -366,6 +380,7 @@ _e_smart_signal_cb_drag_start(void *data, Evas_Object *obj __UNUSED__, const cha
    _e_smart_value_fetch(sd);
    _e_smart_value_limit(sd);
    _e_smart_format_update(sd);
+   if (sd->changing) return;
    if (sd->val != pval)
      evas_object_smart_callback_call(sd->smart_obj, "changed", NULL);
 }
@@ -380,6 +395,7 @@ _e_smart_signal_cb_drag_stop(void *data, Evas_Object *obj __UNUSED__, const char
    _e_smart_value_limit(sd);
    _e_smart_format_update(sd);
    _e_smart_value_update(sd);
+   if (sd->changing) return;
    if (sd->val != pval)
      evas_object_smart_callback_call(sd->smart_obj, "changed", NULL);
 }
@@ -389,6 +405,7 @@ _e_smart_signal_cb_wheel_up(void *data, Evas_Object *obj __UNUSED__, const char
 {
    E_Smart_Data *sd = data;
 
+   if (sd->disabled) return;
    edje_object_part_drag_step(sd->edje_obj, "e.dragable.slider", -0.05, -0.05);
    sd->direction = -1;
 }
@@ -398,6 +415,7 @@ _e_smart_signal_cb_wheel_down(void *data, Evas_Object *obj __UNUSED__, const cha
 {
    E_Smart_Data *sd = data;
 
+   if (sd->disabled) return;
    edje_object_part_drag_step(sd->edje_obj, "e.dragable.slider", 0.05, 0.05);
    sd->direction = 1;
 }
@@ -408,6 +426,7 @@ _e_smart_event_key_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSE
    Evas_Event_Key_Down *ev = event_info;
    E_Smart_Data *sd = data;
 
+   if (sd->disabled) return;
    if ((!strcmp(ev->keyname, "Up")) ||
        (!strcmp(ev->keyname, "KP_Up")) ||
        (!strcmp(ev->keyname, "Left")) ||
@@ -457,7 +476,11 @@ _e_smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj __UNUSED__, void
    Evas_Coord x, y, w, h;
    E_Smart_Data *sd = data;
    double pos;
+   static int in_md = 0;
 
+   if (sd->disabled) return;
+   if (in_md > 0) return;
+   in_md++;
    evas_object_geometry_get(sd->event, &x, &y, &w, &h);
    if (sd->horizontal)
      {
@@ -480,6 +503,7 @@ _e_smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj __UNUSED__, void
    edje_object_message_signal_process(sd->edje_obj); /* really needed or go in infinite loop */
    evas_event_feed_mouse_cancel(e, 0, NULL);
    evas_event_feed_mouse_down(e, 1, EVAS_BUTTON_NONE, ev->timestamp, NULL);
+   in_md--;
 }
 
 static void
@@ -525,7 +549,7 @@ _e_smart_add(Evas_Object *obj)
    evas_object_color_set(sd->event, 0, 0, 0, 0);
    evas_object_pass_events_set(sd->event, EINA_TRUE);
    edje_object_part_swallow(sd->edje_obj, "e.swallow.bar", sd->event);
-   evas_object_smart_member_add(sd->edje_obj, sd->event);
+   evas_object_smart_member_add(sd->event, sd->edje_obj);
 
    edje_object_signal_callback_add(sd->edje_obj, "drag", "*", _e_smart_signal_cb_drag, sd);
    edje_object_signal_callback_add(sd->edje_obj, "drag,start", "*", _e_smart_signal_cb_drag_start, sd);
