@@ -44,13 +44,13 @@ struct _E_Thumb
 /* local subsystem functions */
 static int       _e_ipc_init(void);
 static Eina_Bool _e_ipc_cb_server_add(void *data,
-                                      int   type,
+                                      int type,
                                       void *event);
 static Eina_Bool _e_ipc_cb_server_del(void *data,
-                                      int   type,
+                                      int type,
                                       void *event);
 static Eina_Bool _e_ipc_cb_server_data(void *data,
-                                       int   type,
+                                       int type,
                                        void *event);
 static Eina_Bool _e_cb_timer(void *data);
 static void      _e_thumb_generate(E_Thumb *eth);
@@ -65,7 +65,7 @@ static char _thumbdir[4096] = "";
 
 /* externally accessible functions */
 int
-main(int    argc,
+main(int argc,
      char **argv)
 {
    int i;
@@ -85,11 +85,10 @@ main(int    argc,
         else if (!strncmp(argv[i], "--nice=", 7))
           {
              const char *val;
-             int ret = 0;
 
              val = argv[i] + 7;
              if (*val)
-               ret = nice(atoi(val));
+               nice(atoi(val));
           }
      }
 
@@ -152,7 +151,7 @@ _e_ipc_init(void)
 static Eina_Bool
 _e_ipc_cb_server_add(void *data __UNUSED__,
                      int type   __UNUSED__,
-                     void      *event)
+                     void *event)
 {
    Ecore_Ipc_Event_Server_Add *e;
 
@@ -170,14 +169,14 @@ _e_ipc_cb_server_del(void *data  __UNUSED__,
                      void *event __UNUSED__)
 {
    /* quit now */
-    ecore_main_loop_quit();
-    return ECORE_CALLBACK_PASS_ON;
+   ecore_main_loop_quit();
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
 _e_ipc_cb_server_data(void *data __UNUSED__,
                       int type   __UNUSED__,
-                      void      *event)
+                      void *event)
 {
    Ecore_Ipc_Event_Server_Data *e;
    E_Thumb *eth;
@@ -197,20 +196,20 @@ _e_ipc_cb_server_data(void *data __UNUSED__,
              /* TO a trusted process that WILL send this message properly */
              /* formatted. if the thumbnailer dies anyway - it's not a big loss */
              /* but it is a sign of a bug in e formattign messages maybe */
-                  file = e->data;
-                  key = file + strlen(file) + 1;
-                  if (!key[0]) key = NULL;
-                  eth = calloc(1, sizeof(E_Thumb));
-                  if (eth)
-                    {
-                       eth->objid = e->ref;
-                       eth->w = e->ref_to;
-                       eth->h = e->response;
-                       eth->file = strdup(file);
-                       if (key) eth->key = strdup(key);
-                       _thumblist = eina_list_append(_thumblist, eth);
-                       if (!_timer) _timer = ecore_timer_add(0.001, _e_cb_timer, NULL);
-                    }
+             file = e->data;
+             key = file + strlen(file) + 1;
+             if (!key[0]) key = NULL;
+             eth = calloc(1, sizeof(E_Thumb));
+             if (eth)
+               {
+                  eth->objid = e->ref;
+                  eth->w = e->ref_to;
+                  eth->h = e->response;
+                  eth->file = strdup(file);
+                  if (key) eth->key = strdup(key);
+                  _thumblist = eina_list_append(_thumblist, eth);
+                  if (!_timer) _timer = ecore_timer_add(0.001, _e_cb_timer, NULL);
+               }
           }
         break;
 
@@ -221,8 +220,8 @@ _e_ipc_cb_server_data(void *data __UNUSED__,
              if (eth->objid == e->ref)
                {
                   _thumblist = eina_list_remove_list(_thumblist, l);
-                  if (eth->file) free(eth->file);
-                  if (eth->key) free(eth->key);
+                  free(eth->file);
+                  free(eth->key);
                   free(eth);
                   break;
                }
@@ -254,8 +253,8 @@ _e_cb_timer(void *data __UNUSED__)
         eth = eina_list_data_get(_thumblist);
         _thumblist = eina_list_remove_list(_thumblist, _thumblist);
         _e_thumb_generate(eth);
-        if (eth->file) free(eth->file);
-        if (eth->key) free(eth->key);
+        free(eth->file);
+        free(eth->key);
         free(eth);
 
         if (_thumblist) _timer = ecore_timer_add(0.01, _e_cb_timer, NULL);
@@ -283,7 +282,7 @@ _e_thumb_generate(E_Thumb *eth)
    Evas *evas = NULL, *evas_im = NULL;
    Ecore_Evas *ee = NULL, *ee_im = NULL;
    Evas_Object *im = NULL, *edje = NULL;
-   Eet_File *ef;
+   Eet_File *ef = NULL;
    int iw, ih, alpha, ww, hh;
    const unsigned int *data = NULL;
    time_t mtime_orig, mtime_thumb;
@@ -307,8 +306,10 @@ _e_thumb_generate(E_Thumb *eth)
 
    mtime_orig = ecore_file_mod_time(eth->file);
    mtime_thumb = ecore_file_mod_time(buf);
-   if (mtime_thumb <= mtime_orig)
+   while (mtime_thumb <= mtime_orig)
      {
+        unsigned int *data1;
+
         ecore_file_mkdir(dbuf);
 
         edje_file_cache_set(0);
@@ -347,7 +348,7 @@ _e_thumb_generate(E_Thumb *eth)
                   evas_object_show(edje);
                }
           }
-        else
+        else if (evas_object_image_extension_can_load_get(ext))
           {
              im = evas_object_image_add(evas);
              evas_object_image_load_size_set(im, eth->w, eth->h);
@@ -367,155 +368,151 @@ _e_thumb_generate(E_Thumb *eth)
                   evas_object_image_fill_set(im, 0, 0, ww, hh);
                }
           }
+        else
+          goto end;
+        ecore_evas_alpha_set(ee, alpha);
         evas_object_move(im, 0, 0);
         evas_object_resize(im, ww, hh);
         ecore_evas_resize(ee, ww, hh);
         evas_object_show(im);
-        if (ww > 0)
+        if (ww <= 0) goto end;
+        data = ecore_evas_buffer_pixels_get(ee);
+        if (!data) goto end;
+        ef = eet_open(buf, EET_FILE_MODE_WRITE);
+        if (!ef) goto end;
+        eet_write(ef, "/thumbnail/orig_file",
+                  eth->file, strlen(eth->file), 1);
+        if (eth->key)
+          eet_write(ef, "/thumbnail/orig_key",
+                    eth->key, strlen(eth->key), 1);
+        eet_data_image_write(ef, "/thumbnail/data",
+                             (void *)data, ww, hh, alpha,
+                             0, 91, 1);
+        ww = 4; hh = 4;
+        evas_object_image_fill_set(im, 0, 0, ww, hh);
+        evas_object_resize(im, ww, hh);
+        ecore_evas_resize(ee, ww, hh);
+        data = ecore_evas_buffer_pixels_get(ee);
+        if (!data) goto end;
+
+        data1 = malloc(ww * hh * sizeof(unsigned int));
+        memcpy(data1, data, ww * hh * sizeof(unsigned int));
+        ww = 2; hh = 2;
+        evas_object_image_fill_set(im, 0, 0, ww, hh);
+        evas_object_resize(im, ww, hh);
+        ecore_evas_resize(ee, ww, hh);
+        data = ecore_evas_buffer_pixels_get(ee);
+        if (data)
           {
+             unsigned int *data2;
+
+             data2 = malloc(ww * hh * sizeof(unsigned int));
+             memcpy(data2, data, ww * hh * sizeof(unsigned int));
+             ww = 1; hh = 1;
+             evas_object_image_fill_set(im, 0, 0, ww, hh);
+             evas_object_resize(im, ww, hh);
+             ecore_evas_resize(ee, ww, hh);
              data = ecore_evas_buffer_pixels_get(ee);
              if (data)
                {
-                  ef = eet_open(buf, EET_FILE_MODE_WRITE);
-                  if (ef)
-                    {
-                       eet_write(ef, "/thumbnail/orig_file",
-                                 eth->file, strlen(eth->file), 1);
-                       if (eth->key)
-                         eet_write(ef, "/thumbnail/orig_key",
-                                   eth->key, strlen(eth->key), 1);
-                       eet_data_image_write(ef, "/thumbnail/data",
-                                            (void *)data, ww, hh, alpha,
-                                            0, 91, 1);
-                       ww = 4; hh = 4;
-                       evas_object_image_fill_set(im, 0, 0, ww, hh);
-                       evas_object_resize(im, ww, hh);
-                       ecore_evas_resize(ee, ww, hh);
-                       data = ecore_evas_buffer_pixels_get(ee);
-                       if (data)
-                         {
-                            unsigned int *data1;
+                  unsigned int *data3;
+                  unsigned char id2[(21 * 4) + 1];
+                  int n, i;
+                  int hi, si, vi;
+                  float h, s, v;
+                  const int pat2[4] =
+                  {
+                     0, 3, 1, 2
+                  };
+                  const int pat1[16] =
+                  {
+                     5, 10, 6, 9,
+                     0, 15, 3, 12,
+                     1, 14, 7, 8,
+                     4, 11, 2, 13
+                  };
 
-                            data1 = malloc(ww * hh * sizeof(unsigned int));
-                            memcpy(data1, data, ww * hh * sizeof(unsigned int));
-                            ww = 2; hh = 2;
-                            evas_object_image_fill_set(im, 0, 0, ww, hh);
-                            evas_object_resize(im, ww, hh);
-                            ecore_evas_resize(ee, ww, hh);
-                            data = ecore_evas_buffer_pixels_get(ee);
-                            if (data)
-                              {
-                                 unsigned int *data2;
-
-                                 data2 = malloc(ww * hh * sizeof(unsigned int));
-                                 memcpy(data2, data, ww * hh * sizeof(unsigned int));
-                                 ww = 1; hh = 1;
-                                 evas_object_image_fill_set(im, 0, 0, ww, hh);
-                                 evas_object_resize(im, ww, hh);
-                                 ecore_evas_resize(ee, ww, hh);
-                                 data = ecore_evas_buffer_pixels_get(ee);
-                                 if (data)
-                                   {
-                                      unsigned int *data3;
-                                      unsigned char id2[(21 * 4) + 1];
-                                      int n, i;
-                                      int hi, si, vi;
-                                      float h, s, v;
-                                      const int pat2[4] =
-                                      {
-                                         0, 3, 1, 2
-                                      };
-                                      const int pat1[16] =
-                                      {
-                                         5, 10, 6, 9,
-                                         0, 15, 3, 12,
-                                         1, 14, 7, 8,
-                                         4, 11, 2, 13
-                                      };
-
-                                      data3 = malloc(ww * hh * sizeof(unsigned int));
-                                      memcpy(data3, data, ww * hh * sizeof(unsigned int));
-     // sort_id
-                                      n = 0;
+                  /* ww = hh = 1 here */
+                  data3 = malloc(sizeof(unsigned int));
+                  memcpy(data3, data, sizeof(unsigned int));
+                  // sort_id
+                  n = 0;
 #define A(v) (((v) >> 24) & 0xff)
 #define R(v) (((v) >> 16) & 0xff)
 #define G(v) (((v) >> 8) & 0xff)
 #define B(v) (((v)) & 0xff)
 #define HSV(p)                                         \
-  evas_color_rgb_to_hsv(R(p), G(p), B(p), &h, &s, &v); \
-  hi = 20 * (h / 360.0);                               \
-  si = 20 * s;                                         \
-  vi = 20 * v;                                         \
-  if (si < 2) hi = 25;
+evas_color_rgb_to_hsv(R(p), G(p), B(p), &h, &s, &v); \
+hi = 20 * (h / 360.0);                               \
+si = 20 * s;                                         \
+vi = 20 * v;                                         \
+if (si < 2) hi = 25;
 #define SAVEHSV(h, s, v) \
-  id2[n++] = 'a' + h;     \
-  id2[n++] = 'a' + v;     \
-  id2[n++] = 'a' + s;
+id2[n++] = 'a' + h;    \
+id2[n++] = 'a' + v;    \
+id2[n++] = 'a' + s;
 #define SAVEX(x) \
-  id2[n++] = 'a' + x;
+id2[n++] = 'a' + x;
 #if 0
-                                      HSV(data3[0]);
-                                      SAVEHSV(hi, si, vi);
-                                      for (i = 0; i < 4; i++)
-                                        {
-                                           HSV(data2[pat2[i]]);
-                                           SAVEHSV(hi, si, vi);
-                                        }
-                                      for (i = 0; i < 16; i++)
-                                        {
-                                           HSV(data1[pat1[i]]);
-                                           SAVEHSV(hi, si, vi);
-                                        }
-#else
-                                      HSV(data3[0]);
-                                      SAVEX(hi);
-                                      for (i = 0; i < 4; i++)
-                                        {
-                                           HSV(data2[pat2[i]]);
-                                           SAVEX(hi);
-                                        }
-                                      for (i = 0; i < 16; i++)
-                                        {
-                                           HSV(data1[pat1[i]]);
-                                           SAVEX(hi);
-                                        }
-                                      HSV(data3[0]);
-                                      SAVEX(vi);
-                                      for (i = 0; i < 4; i++)
-                                        {
-                                           HSV(data2[pat2[i]]);
-                                           SAVEX(vi);
-                                        }
-                                      for (i = 0; i < 16; i++)
-                                        {
-                                           HSV(data1[pat1[i]]);
-                                           SAVEX(vi);
-                                        }
-                                      HSV(data3[0]);
-                                      SAVEX(si);
-                                      for (i = 0; i < 4; i++)
-                                        {
-                                           HSV(data2[pat2[i]]);
-                                           SAVEX(si);
-                                        }
-                                      for (i = 0; i < 16; i++)
-                                        {
-                                           HSV(data1[pat1[i]]);
-                                           SAVEX(si);
-                                        }
-#endif
-                                      id2[n++] = 0;
-                                      eet_write(ef, "/thumbnail/sort_id", id2, n, 1);
-                                      free(data3);
-                                   }
-                                 free(data2);
-                              }
-                            free(data1);
-                         }
-                       eet_close(ef);
+                  HSV(data3[0]);
+                  SAVEHSV(hi, si, vi);
+                  for (i = 0; i < 4; i++)
+                    {
+                       HSV(data2[pat2[i]]);
+                       SAVEHSV(hi, si, vi);
                     }
+                  for (i = 0; i < 16; i++)
+                    {
+                       HSV(data1[pat1[i]]);
+                       SAVEHSV(hi, si, vi);
+                    }
+#else
+                  HSV(data3[0]);
+                  SAVEX(hi);
+                  for (i = 0; i < 4; i++)
+                    {
+                       HSV(data2[pat2[i]]);
+                       SAVEX(hi);
+                    }
+                  for (i = 0; i < 16; i++)
+                    {
+                       HSV(data1[pat1[i]]);
+                       SAVEX(hi);
+                    }
+                  HSV(data3[0]);
+                  SAVEX(vi);
+                  for (i = 0; i < 4; i++)
+                    {
+                       HSV(data2[pat2[i]]);
+                       SAVEX(vi);
+                    }
+                  for (i = 0; i < 16; i++)
+                    {
+                       HSV(data1[pat1[i]]);
+                       SAVEX(vi);
+                    }
+                  HSV(data3[0]);
+                  SAVEX(si);
+                  for (i = 0; i < 4; i++)
+                    {
+                       HSV(data2[pat2[i]]);
+                       SAVEX(si);
+                    }
+                  for (i = 0; i < 16; i++)
+                    {
+                       HSV(data1[pat1[i]]);
+                       SAVEX(si);
+                    }
+#endif
+                  id2[n++] = 0;
+                  eet_write(ef, "/thumbnail/sort_id", id2, n, 1);
+                  free(data3);
                }
+             free(data2);
           }
+        free(data1);
+end:
+        if (ef) eet_close(ef);
 
         /* will free all */
         if (edje) evas_object_del(edje);
@@ -524,6 +521,7 @@ _e_thumb_generate(E_Thumb *eth)
           evas_object_del(im);
         ecore_evas_free(ee);
         eet_clearcache();
+        break;
      }
    /* send back path to thumb */
    ecore_ipc_server_send(_e_ipc_server, 5, 2, eth->objid, 0, 0, buf, strlen(buf) + 1);
