@@ -5,19 +5,26 @@ struct _E_Widget_Data
 {
    Evas_Object *text;
 };
-   
+
 /* local subsystem functions */
-static void _e_wid_del_hook(Evas_Object *obj); 
+static void _e_wid_del_hook(Evas_Object *obj);
 static void _e_wid_disable_hook(Evas_Object *obj);
 
 /* externally accessible functions */
+/**  
+ * Creates a new label widget 
+ *
+ * @param evas pointer
+ * @param text to assign to the widget
+ * @return the label widget
+ */
 EAPI Evas_Object *
 e_widget_label_add(Evas *evas, const char *label)
 {
    Evas_Object *obj, *o;
    Evas_Coord mw, mh;
    E_Widget_Data *wd;
-   
+
    obj = e_widget_add(evas);
    e_widget_del_hook_set(obj, _e_wid_del_hook);
    e_widget_disable_hook_set(obj, _e_wid_disable_hook);
@@ -35,10 +42,16 @@ e_widget_label_add(Evas *evas, const char *label)
    e_widget_size_min_set(obj, mw, mh);
    e_widget_sub_object_add(obj, o);
    e_widget_resize_object_set(obj, o);
-      
+
    return obj;
 }
 
+/**  
+ * Changes the text for the label 
+ *
+ * @param the label widget
+ * @param text to assign to the widget
+ */
 EAPI void
 e_widget_label_text_set(Evas_Object *obj, const char *text)
 {
@@ -49,6 +62,7 @@ e_widget_label_text_set(Evas_Object *obj, const char *text)
    edje_object_part_text_set(wd->text, "e.text.label", text);
    edje_object_size_min_calc(wd->text, &mw, &mh);
    edje_extern_object_min_size_set(wd->text, mw, mh);
+   e_widget_size_min_set(obj, mw, mh);
 }
 
 static void
@@ -64,11 +78,10 @@ static void
 _e_wid_disable_hook(Evas_Object *obj)
 {
    E_Widget_Data *wd;
-   
+
    wd = e_widget_data_get(obj);
    if (e_widget_disabled_get(obj))
      edje_object_signal_emit(wd->text, "e,state,disabled", "e");
    else
      edje_object_signal_emit(wd->text, "e,state,enabled", "e");
 }
-
