@@ -1,17 +1,4 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <limits.h>
+#include "e.h"
 
 #ifdef __FreeBSD__
 # include <sys/ioctl.h>
@@ -30,16 +17,6 @@
 # include <ps/IOPowerSources.h>
 #endif
 
-#include <Eina.h>
-#include <Ecore.h>
-#include <Ecore_File.h>
-#include <Ecore_Con.h>
-
-/* define MAX for all OS, instead of plenty #include */
-#ifndef MAX
-# define MAX(a, b) ((a) < (b) ? (b) : (a))
-#endif
-
 /* supported batery system schemes - irrespective of OS */
 #define CHECK_NONE                   0
 #define CHECK_ACPI                   1
@@ -87,7 +64,7 @@ str_file_get(const char *file)
    FILE *f = fopen(file, "r");
    if (f)
      {
-        char buf[PATH_MAX];
+        char buf[4096];
         char *str = fgets(buf, sizeof(buf), f);
         if (str)
           {
@@ -111,7 +88,8 @@ int_get(const char *buf)
    const char *p = strchr(buf, ':');
    if (!p) return 0;
    p++;
-   while (*p == ' ') p++;
+   while (*p == ' ')
+     p++;
    return atoi(p);
 }
 
@@ -124,10 +102,12 @@ str_get(const char *buf)
 
    if (!p) return NULL;
    p++;
-   while (*p == ' ') p++;
+   while (*p == ' ')
+     p++;
 
    q = p + strlen(p) - 1;
-   while ((q > p) && ((*q == ' ') || (*q == '\n'))) q--;
+   while ((q > p) && ((*q == ' ') || (*q == '\n')))
+     q--;
 
    if (q < p) return NULL;
    q++;
@@ -139,7 +119,7 @@ str_get(const char *buf)
 }
 
 static char *
-file_str_entry_get(FILE       *f,
+file_str_entry_get(FILE *f,
                    const char *entry)
 {
    char buf[4096];
@@ -269,8 +249,6 @@ static void
 bsd_apm_check(void)
 {
    int ac_stat, bat_stat, bat_val, time_val;
-   char buf[4096];
-   int hours, minutes;
    int apm_fd = -1;
    struct apm_info info;
 
@@ -283,13 +261,18 @@ bsd_apm_check(void)
    if ((apm_fd != -1) && (ioctl(apm_fd, APMIO_GETINFO, &info) != -1))
      {
         /* set values */
-         ac_stat = info.ai_acline;
-         bat_stat = info.ai_batt_stat;
-         bat_val = info.ai_batt_life;
-         time_val = info.ai_batt_time;
+        ac_stat = info.ai_acline;
+        bat_stat = info.ai_batt_stat;
+        bat_val = info.ai_batt_life;
+        time_val = info.ai_batt_time;
+     }
+   else
+     {
+        if (apm_fd != -1) close(apm_fd);
+        return;
      }
-   else return;
 
+   close(apm_fd);
    if (info.ai_batteries == 1) /* ai_batteries == 1 means NO battery,
                                * ai_batteries == 2 means 1 battery */
      {
@@ -349,7 +332,6 @@ darwin_check(void)
    const void *values;
    int device_num, device_count;
    int currentval = 0, maxval = 0;
-   char buf[4096];
    CFTypeRef blob;
    CFArrayRef sources;
    CFDictionaryRef device_dict;
@@ -420,7 +402,7 @@ darwin_check(void)
 /* new linux power class api to get power info - brand new and this code
  * may have bugs, but it is a good attempt to get it right */
 #if 0
-static Eina_Bool linux_sys_class_power_supply_cb_event_fd_active(void             *data,
+static Eina_Bool linux_sys_class_power_supply_cb_event_fd_active(void *data,
                                                                  Ecore_Fd_Handler *fd_handler);
 static void      linux_sys_class_power_supply_check(void);
 #endif
@@ -476,8 +458,8 @@ linux_sys_class_power_supply_cb_re_init(void *data)
 //	     if (sysev->fd_handler)
 //	       ecore_main_fd_handler_del(sysev->fd_handler);
 //	     if (sysev->fd >= 0) close(sysev->fd);
-                   free(sysev->name);
-                   free(sysev);
+             free(sysev->name);
+             free(sysev);
           }
      }
    linux_sys_class_power_supply_init();
@@ -486,7 +468,7 @@ linux_sys_class_power_supply_cb_re_init(void *data)
 }
 
 static Eina_Bool
-linux_sys_class_power_supply_cb_event_fd_active(void             *data,
+linux_sys_class_power_supply_cb_event_fd_active(void *data,
                                                 Ecore_Fd_Handler *fd_handler)
 {
    Sys_Class_Power_Supply_Uevent *sysev;
@@ -802,7 +784,7 @@ linux_sys_class_power_supply_check(void)
              else if (sysev->basis == BASIS_ENERGY)
                snprintf(buf, sizeof(buf), "%s/%s/energy_now", dir, name);
              else if (sysev->basis == BASIS_VOLTAGE)
-                snprintf(buf, sizeof(buf), "%s/%s/voltage_now", dir, name);
+               snprintf(buf, sizeof(buf), "%s/%s/voltage_now", dir, name);
              pwr_now = int_file_get(buf);
              pwr_empty = sysev->basis_empty;
              pwr_full = sysev->basis_full;
@@ -824,9 +806,9 @@ linux_sys_class_power_supply_check(void)
                   else
                     {
                        if (pwr_now < 0)
-                          pwr_now = (((long long)capacity * ((long long)pwr_full - (long long)pwr_empty)) / 100) + pwr_empty;
+                         pwr_now = (((long long)capacity * ((long long)pwr_full - (long long)pwr_empty)) / 100) + pwr_empty;
                     }
-                  
+
                   if (sysev->present) have_battery = 1;
                   if (charging)
                     {
@@ -835,13 +817,13 @@ linux_sys_class_power_supply_check(void)
                        if (time_to_full >= 0)
                          {
                             if (time_to_full > time_left)
-                               time_left = time_to_full;
+                              time_left = time_to_full;
                          }
                        else
                          {
                             if (current == 0) time_left = 0;
                             else if (current < 0)
-                               time_left = -1;
+                              time_left = -1;
                             else
                               {
                                  pwr = (((long long)pwr_full - (long long)pwr_now) * 3600) / -current;
@@ -892,16 +874,16 @@ linux_sys_class_power_supply_check(void)
  * power class api to poll for power stuff
  */
 static Eina_Bool linux_acpi_cb_acpid_add(void *data,
-                                         int   type,
+                                         int type,
                                          void *event);
 static Eina_Bool linux_acpi_cb_acpid_del(void *data,
-                                         int   type,
+                                         int type,
                                          void *event);
 static Eina_Bool linux_acpi_cb_acpid_data(void *data,
-                                          int   type,
+                                          int type,
                                           void *event);
-static void linux_acpi_init(void);
-static void linux_acpi_check(void);
+static void      linux_acpi_init(void);
+static void      linux_acpi_check(void);
 
 static int acpi_max_full = -1;
 static int acpi_max_design = -1;
@@ -997,32 +979,32 @@ linux_acpi_cb_event_fd_active(void *data        __UNUSED__,
 static void
 linux_acpi_init(void)
 {
-   Eina_List *powers;
-   Eina_List *bats;
+   Eina_Iterator *powers;
+   Eina_Iterator *bats;
 
-   bats = ecore_file_ls("/proc/acpi/battery");
+   bats = eina_file_direct_ls("/proc/acpi/battery");
    if (bats)
      {
-        char *name;
+        Eina_File_Direct_Info *info;
 
         have_power = 0;
-        powers = ecore_file_ls("/proc/acpi/ac_adapter");
+        powers = eina_file_direct_ls("/proc/acpi/ac_adapter");
         if (powers)
           {
-             char *name;
-
-             EINA_LIST_FREE(powers, name)
+             EINA_ITERATOR_FOREACH(powers, info)
                {
-                  char buf[4096];
+                  char buf[PATH_MAX];
                   FILE *f;
 
-                  snprintf(buf, sizeof(buf), "/proc/acpi/ac_adapter/%s/state", name);
+                  if (info->name_length + sizeof("/state") >= sizeof(buf)) continue;
+                  strcpy(buf, info->path);
+                  strcat(buf, "/state");
                   f = fopen(buf, "r");
                   if (f)
                     {
                        char *tmp;
 
-     /* state */
+                       /* state */
                        tmp = fgets(buf, sizeof(buf), f);
                        if (tmp) tmp = str_get(tmp);
                        if (tmp)
@@ -1030,27 +1012,28 @@ linux_acpi_init(void)
                             if (!strcmp(tmp, "on-line")) have_power = 1;
                             free(tmp);
                          }
+                       fclose(f);
                     }
-
-                  free(name);
                }
+             eina_iterator_free(powers);
           }
 
         have_battery = 0;
         acpi_max_full = 0;
         acpi_max_design = 0;
-        EINA_LIST_FREE(bats, name)
+        EINA_ITERATOR_FOREACH(bats, info)
           {
              char buf[4096];
              FILE *f;
 
-             snprintf(buf, sizeof(buf), "/proc/acpi/battery/%s/info", name);
+             strcpy(buf, info->path);
+             strcat(buf, "/info");
              f = fopen(buf, "r");
              if (f)
                {
                   char *tmp;
 
-     /* present */
+                  /* present */
                   tmp = fgets(buf, sizeof(buf), f);
                   if (tmp) tmp = str_get(tmp);
                   if (tmp)
@@ -1058,7 +1041,7 @@ linux_acpi_init(void)
                        if (!strcmp(tmp, "yes")) have_battery = 1;
                        free(tmp);
                     }
-     /* design cap */
+                  /* design cap */
                   tmp = fgets(buf, sizeof(buf), f);
                   if (tmp) tmp = str_get(tmp);
                   if (tmp)
@@ -1066,7 +1049,7 @@ linux_acpi_init(void)
                        if (strcmp(tmp, "unknown")) acpi_max_design += atoi(tmp);
                        free(tmp);
                     }
-     /* last full cap */
+                  /* last full cap */
                   tmp = fgets(buf, sizeof(buf), f);
                   if (tmp) tmp = str_get(tmp);
                   if (tmp)
@@ -1076,9 +1059,9 @@ linux_acpi_init(void)
                     }
                   fclose(f);
                }
-
-             free(name);
           }
+
+        eina_iterator_free(bats);
      }
    if (!acpid)
      {
@@ -1374,17 +1357,17 @@ fclose_and_continue:
                   fclose(f);
                   if (!current)
                     {
-     /* Neither charging nor discharging */
+                       /* Neither charging nor discharging */
                     }
                   else if (!ac)
                     {
-     /* When on dc, we are discharging */
-                        seconds += timeleft;
+                       /* When on dc, we are discharging */
+                       seconds += timeleft;
                     }
                   else
                     {
-     /* Charging - works in parallel */
-                        seconds = MAX(timeleft, seconds);
+                       /* Charging - works in parallel */
+                       seconds = MAX(timeleft, seconds);
                     }
                }
 
@@ -1556,7 +1539,7 @@ poll_cb(void *data __UNUSED__)
 }
 
 int
-main(int   argc,
+main(int argc,
      char *argv[])
 {
    if (argc != 2)
