@@ -24,7 +24,6 @@ typedef struct _Match_Config
 
 struct _E_Config_Dialog_Data
 {
-   int         use_shadow;
    int         engine;
    int         indirect;
    int         texture_from_pixmap;
@@ -34,6 +33,7 @@ struct _E_Config_Dialog_Data
    int         loose_sync;
    int         grab;
    int         vsync;
+   int         swap_mode;
 
    const char *shadow_style;
 
@@ -69,16 +69,21 @@ struct _E_Config_Dialog_Data
 
 /* Protos */
 static void        *_create_data(E_Config_Dialog *cfd);
-static void         _free_data(E_Config_Dialog      *cfd,
+static void         _free_data(E_Config_Dialog *cfd,
                                E_Config_Dialog_Data *cfdata);
-static Evas_Object *_basic_create_widgets(E_Config_Dialog      *cfd,
-                                          Evas                 *evas,
+static Evas_Object *_basic_create_widgets(E_Config_Dialog *cfd,
+                                          Evas *evas,
                                           E_Config_Dialog_Data *cfdata);
-static int _basic_apply_data(E_Config_Dialog      *cfd,
-                             E_Config_Dialog_Data *cfdata);
+static int          _basic_apply_data(E_Config_Dialog *cfd,
+                                      E_Config_Dialog_Data *cfdata);
+static Evas_Object *_advanced_create_widgets(E_Config_Dialog *cfd,
+                                          Evas *evas,
+                                          E_Config_Dialog_Data *cfdata);
+static int          _advanced_apply_data(E_Config_Dialog *cfd,
+                                      E_Config_Dialog_Data *cfdata);
 
 E_Config_Dialog *
-e_int_config_comp_module(E_Container       *con,
+e_int_config_comp_module(E_Container *con,
                          const char *params __UNUSED__)
 {
    E_Config_Dialog *cfd;
@@ -93,7 +98,9 @@ e_int_config_comp_module(E_Container       *con,
    v->free_cfdata = _free_data;
    v->basic.apply_cfdata = _basic_apply_data;
    v->basic.create_widgets = _basic_create_widgets;
-
+   v->advanced.apply_cfdata = _advanced_apply_data;
+   v->advanced.create_widgets = _advanced_create_widgets;
+   
    snprintf(buf, sizeof(buf), "%s/e-module-comp.edj",
             e_module_dir_get(mod->module));
    cfd = e_config_dialog_new(con, _("Composite Settings"),
@@ -103,7 +110,7 @@ e_int_config_comp_module(E_Container       *con,
 }
 
 static void
-_match_dup(Match        *m,
+_match_dup(Match *m,
            Match_Config *m2)
 {
    m2->match = *m;
@@ -124,7 +131,6 @@ _create_data(E_Config_Dialog *cfd)
 
    cfdata = E_NEW(E_Config_Dialog_Data, 1);
 
-   cfdata->use_shadow = _comp_mod->conf->use_shadow;
    cfdata->engine = _comp_mod->conf->engine;
    if ((cfdata->engine != ENGINE_SW) &&
        (cfdata->engine != ENGINE_GL))
@@ -137,6 +143,7 @@ _create_data(E_Config_Dialog *cfd)
    cfdata->loose_sync = _comp_mod->conf->loose_sync;
    cfdata->grab = _comp_mod->conf->grab;
    cfdata->vsync = _comp_mod->conf->vsync;
+   cfdata->swap_mode = _comp_mod->conf->swap_mode;
    if (_comp_mod->conf->shadow_style)
      cfdata->shadow_style = eina_stringshare_add(_comp_mod->conf->shadow_style);
 
@@ -199,10 +206,10 @@ _match_free(Match_Config *m)
    if (m->match.clas) eina_stringshare_del(m->match.clas);
    if (m->match.role) eina_stringshare_del(m->match.role);
    if (m->match.shadow_style) eina_stringshare_del(m->match.shadow_style);
-   if (m->title) free(m->title);
-   if (m->name) free(m->name);
-   if (m->clas) free(m->clas);
-   if (m->role) free(m->role);
+   free(m->title);
+   free(m->name);
+   free(m->clas);
+   free(m->role);
    free(m);
 }
 
@@ -233,28 +240,6 @@ _free_data(E_Config_Dialog *cfd  __UNUSED__,
    free(cfdata);
 }
 
-static void
-_shadow_changed(void            *data,
-                Evas_Object     *obj,
-                void *event_info __UNUSED__)
-{
-   E_Config_Dialog_Data *cfdata = data;
-   Evas_Object *orec0;
-   Eina_List *style_list;
-   const E_Demo_Style_Item *it;
-   const Eina_List *l;
-
-   orec0 = evas_object_name_find(evas_object_evas_get(obj), "style_shadows");
-   style_list = evas_object_data_get(orec0, "list");
-   EINA_LIST_FOREACH(style_list, l, it)
-     {
-        if (cfdata->use_shadow)
-          edje_object_signal_emit(it->preview, "e,state,shadow,on", "e");
-        else
-          edje_object_signal_emit(it->preview, "e,state,shadow,off", "e");
-     }
-}
-
 static Eina_Bool
 _style_demo(void *data)
 {
@@ -304,8 +289,8 @@ _style_demo(void *data)
 
 static void
 _style_selector_del(void *data       __UNUSED__,
-                    Evas            *e,
-                    Evas_Object     *o,
+                    Evas *e,
+                    Evas_Object *o,
                     void *event_info __UNUSED__)
 {
    Eina_List *style_shadows, *style_list;
@@ -346,8 +331,7 @@ _style_selector_del(void *data       __UNUSED__,
 }
 
 static Evas_Object *
-_style_selector(Evas        *evas,
-                int          use_shadow,
+_style_selector(Evas *evas,
                 const char **source)
 {
    Evas_Object *oi, *ob, *oo, *obd, *orec, *oly, *orec0;
@@ -369,7 +353,7 @@ _style_selector(Evas        *evas,
    EINA_LIST_FOREACH(styles, l, style)
      {
         E_Demo_Style_Item *ds_it;
-        char buf[PATH_MAX];
+        char buf[4096];
 
         ds_it = malloc(sizeof(E_Demo_Style_Item));
 
@@ -390,7 +374,7 @@ _style_selector(Evas        *evas,
         e_layout_pack(oly, oo);
         e_layout_child_move(oo, 39, 39);
         e_layout_child_resize(oo, 162, 162);
-        if (use_shadow) edje_object_signal_emit(oo, "e,state,shadow,on", "e");
+        edje_object_signal_emit(oo, "e,state,shadow,on", "e");
         edje_object_signal_emit(oo, "e,state,visible,on", "e");
         evas_object_show(oo);
 
@@ -412,7 +396,7 @@ _style_selector(Evas        *evas,
 
         orec = evas_object_rectangle_add(e_livethumb_evas_get(ob));
         ds_it->client = orec;
-        evas_object_color_set(orec, 255, 255, 255, 255);
+        evas_object_color_set(orec, 0, 0, 0, 128);
         edje_object_part_swallow(obd, "e.swallow.client", orec);
         evas_object_show(orec);
 
@@ -538,10 +522,10 @@ _match_label_get(Match_Config *m)
 }
 
 static void
-_match_ilist_append(Evas_Object  *il,
+_match_ilist_append(Evas_Object *il,
                     Match_Config *m,
-                    int           pos,
-                    int           pre)
+                    int pos,
+                    int pre)
 {
    char *name = _match_label_get(m);
 
@@ -558,7 +542,7 @@ _match_ilist_append(Evas_Object  *il,
 }
 
 static void
-_match_list_up(Eina_List   **list,
+_match_list_up(Eina_List **list,
                Match_Config *m)
 {
    Eina_List *l, *lp;
@@ -572,7 +556,7 @@ _match_list_up(Eina_List   **list,
 }
 
 static void
-_match_list_down(Eina_List   **list,
+_match_list_down(Eina_List **list,
                  Match_Config *m)
 {
    Eina_List *l, *lp;
@@ -586,22 +570,21 @@ _match_list_down(Eina_List   **list,
 }
 
 static void
-_match_list_del(Eina_List   **list,
+_match_list_del(Eina_List **list,
                 Match_Config *m)
 {
-   Eina_List *l, *lp;
+   Eina_List *l;
 
    l = eina_list_data_find_list(*list, m);
    if (!l) return;
-   lp = l->next;
    *list = eina_list_remove_list(*list, l);
    _match_free(m);
 }
 
 static void
-_cb_dialog_resize(void            *data,
+_cb_dialog_resize(void *data,
                   Evas *e          __UNUSED__,
-                  Evas_Object     *obj,
+                  Evas_Object *obj,
                   void *event_info __UNUSED__)
 {
    Evas_Object *bg, *of;
@@ -622,7 +605,7 @@ _edit_ok(void *d1,
          void *d2)
 {
    Match_Config *m = d1;
-   Evas_Object *dia, *bg, *of = d2;
+  Evas_Object *dia, *bg, *of = d2;
    Evas_Object *il;
    char *label;
    int n;
@@ -632,32 +615,28 @@ _edit_ok(void *d1,
    if (m->title)
      {
         if (m->title[0]) m->match.title = eina_stringshare_add(m->title);
-        free(m->title);
-        m->title = NULL;
+        E_FREE(m->title);
      }
    if (m->match.name) eina_stringshare_del(m->match.name);
    m->match.name = NULL;
    if (m->name)
      {
         if (m->name[0]) m->match.name = eina_stringshare_add(m->name);
-        free(m->name);
-        m->name = NULL;
+        E_FREE(m->name);
      }
    if (m->match.clas) eina_stringshare_del(m->match.clas);
    m->match.clas = NULL;
    if (m->clas)
      {
         if (m->clas[0]) m->match.clas = eina_stringshare_add(m->clas);
-        free(m->clas);
-        m->clas = NULL;
+        E_FREE(m->clas);
      }
    if (m->match.role) eina_stringshare_del(m->match.role);
    m->match.role = NULL;
    if (m->role)
      {
         if (m->role[0]) m->match.role = eina_stringshare_add(m->role);
-        free(m->role);
-        m->role = NULL;
+        E_FREE(m->role);
      }
    m->match.borderless = m->borderless;
    m->match.dialog = m->dialog;
@@ -682,10 +661,10 @@ _edit_ok(void *d1,
 }
 
 static void
-_create_edit_frame(E_Config_Dialog      *cfd,
-                   Evas                 *evas,
+_create_edit_frame(E_Config_Dialog *cfd,
+                   Evas *evas,
                    E_Config_Dialog_Data *cfdata,
-                   Match_Config         *m)
+                   Match_Config *m)
 {
    Evas_Object *of, *oi, *lb, *en, *bt, *tb, *tab2, *o, *sf, *li;
    E_Radio_Group *rg;
@@ -925,7 +904,7 @@ _create_edit_frame(E_Config_Dialog      *cfd,
    e_widget_toolbook_page_append(tb, NULL, _("Flags"), tab2,
                                  1, 1, 1, 1, 0.5, 0.0);
 
-   oi = _style_selector(evas, cfdata->use_shadow, &(m->match.shadow_style));
+   oi = _style_selector(evas, &(m->match.shadow_style));
    e_widget_toolbook_page_append(tb, NULL, _("Style"), oi,
                                  1, 1, 1, 1, 0.5, 0.0);
 
@@ -1083,11 +1062,11 @@ _but_edit(void *d1,
 }
 
 static Evas_Object *
-_create_match_editor(E_Config_Dialog             *cfd,
-                     Evas                        *evas,
+_create_match_editor(E_Config_Dialog *cfd,
+                     Evas *evas,
                      E_Config_Dialog_Data *cfdata __UNUSED__,
-                     Eina_List                  **matches,
-                     Evas_Object                **il_ret)
+                     Eina_List **matches,
+                     Evas_Object **il_ret)
 {
    Evas_Object *tab, *il, *bt;
    Match_Config *m;
@@ -1123,15 +1102,15 @@ _create_match_editor(E_Config_Dialog             *cfd,
 }
 
 static Evas_Object *
-_create_styles_toolbook(E_Config_Dialog      *cfd,
-                        Evas                 *evas,
+_create_styles_toolbook(E_Config_Dialog *cfd,
+                        Evas *evas,
                         E_Config_Dialog_Data *cfdata)
 {
    Evas_Object *tb, *oi, *il;
 
    tb = e_widget_toolbook_add(evas, 48 * e_scale, 48 * e_scale);
 
-   oi = _style_selector(evas, cfdata->use_shadow, &(cfdata->shadow_style));
+   oi = _style_selector(evas, &(cfdata->shadow_style));
    e_widget_toolbook_page_append(tb, NULL, _("Default"), oi, 1, 1, 1, 1, 0.5, 0.0);
 
    oi = _create_match_editor(cfd, evas, cfdata, &(cfdata->match.borders), &il);
@@ -1156,13 +1135,15 @@ _create_styles_toolbook(E_Config_Dialog      *cfd,
 }
 
 static Evas_Object *
-_basic_create_widgets(E_Config_Dialog      *cfd,
-                      Evas                 *evas,
-                      E_Config_Dialog_Data *cfdata)
+_advanced_create_widgets(E_Config_Dialog *cfd,
+                         Evas *evas,
+                         E_Config_Dialog_Data *cfdata)
 {
-   Evas_Object *ob, *ol, *ol2, *of, *otb, *oi, *orec0, *tab;
+   Evas_Object *ob,*ol, *of, *otb, *oi, *orec0, *tab;
    E_Radio_Group *rg;
 
+   e_dialog_resizable_set(cfd->dia, 1);
+   
    orec0 = evas_object_rectangle_add(evas);
    evas_object_name_set(orec0, "style_shadows");
 
@@ -1173,11 +1154,9 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
 
    ///////////////////////////////////////////
    ol = e_widget_list_add(evas, 0, 0);
-   ob = e_widget_check_add(evas, _("Shadows"), &(cfdata->use_shadow));
-   evas_object_smart_callback_add(ob, "changed", _shadow_changed, cfdata);
-   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
-   ob = e_widget_check_add(evas, _("Limit framerate"), &(cfdata->lock_fps));
-   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
+// disabled because this is disabled in code for now   
+//   ob = e_widget_check_add(evas, _("Limit framerate"), &(cfdata->lock_fps));
+//   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
    ob = e_widget_check_add(evas, _("Smooth scaling"), &(cfdata->smooth_windows));
    e_widget_list_object_append(ol, ob, 1, 0, 0.5);
 
@@ -1191,8 +1170,6 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
 
    ///////////////////////////////////////////
    ol = e_widget_list_add(evas, 0, 0);
-   ob = e_widget_check_add(evas, _("Sync screen (VBlank)"), &(cfdata->vsync));
-   e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    ob = e_widget_check_add(evas, _("Sync windows"), &(cfdata->efl_sync));
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    ob = e_widget_check_add(evas, _("Loose sync"), &(cfdata->loose_sync));
@@ -1205,11 +1182,12 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    e_widget_toolbook_page_append(otb, NULL, _("Sync"), ol, 0, 0, 0, 0, 0.5, 0.0);
 
+   ///////////////////////////////////////////
    ol = e_widget_list_add(evas, 0, 0);
    rg = e_widget_radio_group_new(&(cfdata->engine));
    ob = e_widget_radio_add(evas, _("Software"), ENGINE_SW, rg);
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
-   if (!getenv("ECORE_X_NO_XLIB")) 
+   if (!getenv("ECORE_X_NO_XLIB"))
      {
         if (ecore_evas_engine_type_supported_get(ECORE_EVAS_ENGINE_OPENGL_X11))
           {
@@ -1218,12 +1196,33 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
 
              of = e_widget_framelist_add(evas, _("OpenGL options"), 0);
              e_widget_framelist_content_align_set(of, 0.5, 0.0);
+             ob = e_widget_check_add(evas, _("Tear-free updates (VSynced)"), &(cfdata->vsync));
+             e_widget_framelist_object_append(of, ob);
              ob = e_widget_check_add(evas, _("Texture from pixmap"), &(cfdata->texture_from_pixmap));
              e_widget_framelist_object_append(of, ob);
-	     ob = e_widget_label_add(evas, _("Ctrl+Alt+Shift+Home resets compositor"));
-	     e_widget_framelist_object_append(of, ob);
-             ob = e_widget_check_add(evas, _("Indirect OpenGL (EXPERIMENTAL)"), &(cfdata->indirect));
-             e_widget_framelist_object_append(of, ob);
+#ifdef ECORE_EVAS_GL_X11_OPT_SWAP_MODE             
+             if ((evas_version->major >= 1) &&
+                 (evas_version->minor >= 7) &&
+                 (evas_version->micro >= 99))
+               {
+                  ob = e_widget_label_add(evas, _("Assume swapping method:"));
+                  e_widget_framelist_object_append(of, ob);
+                  rg = e_widget_radio_group_new(&(cfdata->swap_mode));
+                  ob = e_widget_radio_add(evas, _("Auto"), ECORE_EVAS_GL_X11_SWAP_MODE_AUTO, rg);
+                  e_widget_framelist_object_append(of, ob);
+                  ob = e_widget_radio_add(evas, _("Invalidate (full redraw)"), ECORE_EVAS_GL_X11_SWAP_MODE_FULL, rg);
+                  e_widget_framelist_object_append(of, ob);
+                  ob = e_widget_radio_add(evas, _("Copy from back to front"), ECORE_EVAS_GL_X11_SWAP_MODE_COPY, rg);
+                  e_widget_framelist_object_append(of, ob);
+                  ob = e_widget_radio_add(evas, _("Double buffered swaps"), ECORE_EVAS_GL_X11_SWAP_MODE_DOUBLE, rg);
+                  e_widget_framelist_object_append(of, ob);
+                  ob = e_widget_radio_add(evas, _("Triple buffered swaps"), ECORE_EVAS_GL_X11_SWAP_MODE_TRIPLE, rg);
+                  e_widget_framelist_object_append(of, ob);
+               }
+#endif             
+// lets not offer this anymore             
+//             ob = e_widget_check_add(evas, _("Indirect OpenGL (EXPERIMENTAL)"), &(cfdata->indirect));
+//             e_widget_framelist_object_append(of, ob);
              e_widget_list_object_append(ol, of, 1, 1, 0.5);
           }
      }
@@ -1235,8 +1234,9 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    ob = e_widget_check_add(evas, _("Send dump"), &(cfdata->send_dump));
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
-   ob = e_widget_check_add(evas, _("Don't composite fullscreen"), &(cfdata->nocomp_fs));
+   ob = e_widget_check_add(evas, _("Don't composite fullscreen windows"), &(cfdata->nocomp_fs));
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
+/*   
    ob = e_widget_check_add(evas, _("Keep hidden windows"), &(cfdata->keep_unmapped));
    e_widget_list_object_append(ol, ob, 1, 1, 0.5);
    of = e_widget_frametable_add(evas, _("Maximum hidden pixels"), 0);
@@ -1261,9 +1261,11 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
    ob = e_widget_radio_add(evas, _("256M"), 256 * 1024, rg);
    e_widget_frametable_object_append(of, ob, 2, 2, 1, 1, 1, 1, 0, 0);
    e_widget_list_object_append(ol, of, 1, 1, 0.5);
+ */
    e_widget_toolbook_page_append(otb, NULL, _("Memory"), ol, 0, 0, 0, 0, 0.5, 0.0);
 
    ///////////////////////////////////////////
+/*   
    ol = e_widget_list_add(evas, 0, 0);
    ol2 = e_widget_list_add(evas, 1, 1);
    of = e_widget_framelist_add(evas, _("Min hidden"), 0);
@@ -1304,7 +1306,7 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
    e_widget_list_object_append(ol2, of, 1, 1, 0.5);
    e_widget_list_object_append(ol, ol2, 1, 1, 0.5);
    e_widget_toolbook_page_append(otb, NULL, _("Timeouts"), ol, 0, 0, 0, 0, 0.5, 0.0);
-
+ */
    ///////////////////////////////////////////
    ol = e_widget_list_add(evas, 0, 0);
 
@@ -1319,16 +1321,16 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
    of = e_widget_frametable_add(evas, _("Corner"), 0);
    e_widget_frametable_content_align_set(of, 0.5, 0.5);
    rg = e_widget_radio_group_new(&(cfdata->fps_corner));
-   ob = e_widget_radio_icon_add(evas, "Top Left", "preferences-position-top-left",
+   ob = e_widget_radio_icon_add(evas, _("Top Left"), "preferences-position-top-left",
                                 24, 24, 0, rg);
    e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 1, 1, 1);
-   ob = e_widget_radio_icon_add(evas, "Top Right", "preferences-position-top-right",
+   ob = e_widget_radio_icon_add(evas, _("Top Right"), "preferences-position-top-right",
                                 24, 24, 1, rg);
    e_widget_frametable_object_append(of, ob, 1, 0, 1, 1, 1, 1, 1, 1);
-   ob = e_widget_radio_icon_add(evas, "Bottom Left", "preferences-position-bottom-left",
+   ob = e_widget_radio_icon_add(evas, _("Bottom Left"), "preferences-position-bottom-left",
                                 24, 24, 2, rg);
    e_widget_frametable_object_append(of, ob, 0, 1, 1, 1, 1, 1, 1, 1);
-   ob = e_widget_radio_icon_add(evas, "Bottom Right", "preferences-position-bottom-right",
+   ob = e_widget_radio_icon_add(evas, _("Bottom Right"), "preferences-position-bottom-right",
                                 24, 24, 3, rg);
    e_widget_frametable_object_append(of, ob, 1, 1, 1, 1, 1, 1, 1, 1);
    e_widget_list_object_append(ol, of, 1, 1, 0.5);
@@ -1337,7 +1339,6 @@ _basic_create_widgets(E_Config_Dialog      *cfd,
 
    e_widget_toolbook_page_show(otb, 0);
 
-   e_dialog_resizable_set(cfd->dia, 1);
 
    e_widget_table_object_append(tab, otb, 0, 0, 1, 1, 1, 1, 1, 1);
    return tab;
@@ -1361,7 +1362,7 @@ _match_list_free(Eina_List *list)
 
 static void
 _match_dup2(Match_Config *m2,
-            Match        *m)
+            Match *m)
 {
    *m = m2->match;
    if (m->title) m->title = eina_stringshare_add(m->title);
@@ -1372,11 +1373,191 @@ _match_dup2(Match_Config *m2,
 }
 
 static int
-_basic_apply_data(E_Config_Dialog *cfd  __UNUSED__,
-                  E_Config_Dialog_Data *cfdata)
+_advanced_apply_data(E_Config_Dialog *cfd  __UNUSED__,
+                     E_Config_Dialog_Data *cfdata)
 {
-   if ((cfdata->use_shadow != _comp_mod->conf->use_shadow) ||
-       (cfdata->lock_fps != _comp_mod->conf->lock_fps) ||
+   if ((cfdata->lock_fps != _comp_mod->conf->lock_fps) ||
+       (cfdata->smooth_windows != _comp_mod->conf->smooth_windows) ||
+       (cfdata->grab != _comp_mod->conf->grab) ||
+       (cfdata->keep_unmapped != _comp_mod->conf->keep_unmapped) ||
+       (cfdata->nocomp_fs != _comp_mod->conf->nocomp_fs) ||
+       (cfdata->shadow_style != _comp_mod->conf->shadow_style) ||
+       (cfdata->max_unmapped_pixels != _comp_mod->conf->max_unmapped_pixels) ||
+       (cfdata->max_unmapped_time != _comp_mod->conf->max_unmapped_time) ||
+       (cfdata->min_unmapped_time != _comp_mod->conf->min_unmapped_time) ||
+       (cfdata->send_flush != _comp_mod->conf->send_flush) ||
+       (cfdata->send_dump != _comp_mod->conf->send_dump) ||
+       (cfdata->fps_show != _comp_mod->conf->fps_show) ||
+       (cfdata->fps_corner != _comp_mod->conf->fps_corner) ||
+       (cfdata->fps_average_range != _comp_mod->conf->fps_average_range) ||
+       (cfdata->first_draw_delay != _comp_mod->conf->first_draw_delay) ||
+       (cfdata->match.changed)
+       )
+     {
+        if (cfdata->match.changed)
+          {
+             Eina_List *l;
+             Match *m;
+             Match_Config *m2;
+
+             _match_list_free(_comp_mod->conf->match.popups);
+             _match_list_free(_comp_mod->conf->match.borders);
+             _match_list_free(_comp_mod->conf->match.overrides);
+             _match_list_free(_comp_mod->conf->match.menus);
+
+             _comp_mod->conf->match.popups = NULL;
+             _comp_mod->conf->match.borders = NULL;
+             _comp_mod->conf->match.overrides = NULL;
+             _comp_mod->conf->match.menus = NULL;
+
+             EINA_LIST_FOREACH(cfdata->match.popups, l, m2)
+               {
+                  m = E_NEW(Match, 1);
+                  _match_dup2(m2, m);
+                  _comp_mod->conf->match.popups =
+                    eina_list_append(_comp_mod->conf->match.popups, m);
+               }
+             EINA_LIST_FOREACH(cfdata->match.borders, l, m2)
+               {
+                  m = E_NEW(Match, 1);
+                  _match_dup2(m2, m);
+                  _comp_mod->conf->match.borders =
+                    eina_list_append(_comp_mod->conf->match.borders, m);
+               }
+             EINA_LIST_FOREACH(cfdata->match.overrides, l, m2)
+               {
+                  m = E_NEW(Match, 1);
+                  _match_dup2(m2, m);
+                  _comp_mod->conf->match.overrides =
+                    eina_list_append(_comp_mod->conf->match.overrides, m);
+               }
+             EINA_LIST_FOREACH(cfdata->match.menus, l, m2)
+               {
+                  m = E_NEW(Match, 1);
+                  _match_dup2(m2, m);
+                  _comp_mod->conf->match.menus =
+                    eina_list_append(_comp_mod->conf->match.menus, m);
+               }
+             cfdata->match.changed = 0;
+          }
+        _comp_mod->conf->lock_fps = cfdata->lock_fps;
+        _comp_mod->conf->smooth_windows = cfdata->smooth_windows;
+        _comp_mod->conf->grab = cfdata->grab;
+        _comp_mod->conf->keep_unmapped = cfdata->keep_unmapped;
+        _comp_mod->conf->nocomp_fs = cfdata->nocomp_fs;
+        _comp_mod->conf->max_unmapped_pixels = cfdata->max_unmapped_pixels;
+        _comp_mod->conf->max_unmapped_time = cfdata->max_unmapped_time;
+        _comp_mod->conf->min_unmapped_time = cfdata->min_unmapped_time;
+        _comp_mod->conf->send_flush = cfdata->send_flush;
+        _comp_mod->conf->send_dump = cfdata->send_dump;
+        _comp_mod->conf->fps_show = cfdata->fps_show;
+        _comp_mod->conf->fps_corner = cfdata->fps_corner;
+        _comp_mod->conf->fps_average_range = cfdata->fps_average_range;
+        _comp_mod->conf->first_draw_delay = cfdata->first_draw_delay;
+        if (_comp_mod->conf->shadow_style)
+          eina_stringshare_del(_comp_mod->conf->shadow_style);
+        _comp_mod->conf->shadow_style = NULL;
+        if (cfdata->shadow_style)
+          _comp_mod->conf->shadow_style = eina_stringshare_add(cfdata->shadow_style);
+        e_mod_comp_shadow_set();
+     }
+   if ((cfdata->engine != _comp_mod->conf->engine) ||
+       (cfdata->indirect != _comp_mod->conf->indirect) ||
+       (cfdata->texture_from_pixmap != _comp_mod->conf->texture_from_pixmap) ||
+       (cfdata->efl_sync != _comp_mod->conf->efl_sync) ||
+       (cfdata->loose_sync != _comp_mod->conf->loose_sync) ||
+       (cfdata->vsync != _comp_mod->conf->vsync) ||
+       (cfdata->swap_mode != _comp_mod->conf->swap_mode))
+     {
+        E_Action *a;
+
+        _comp_mod->conf->engine = cfdata->engine;
+        _comp_mod->conf->indirect = cfdata->indirect;
+        _comp_mod->conf->texture_from_pixmap = cfdata->texture_from_pixmap;
+        _comp_mod->conf->efl_sync = cfdata->efl_sync;
+        _comp_mod->conf->loose_sync = cfdata->loose_sync;
+        _comp_mod->conf->vsync = cfdata->vsync;
+        _comp_mod->conf->swap_mode = cfdata->swap_mode;
+
+        a = e_action_find("restart");
+        if ((a) && (a->func.go)) a->func.go(NULL, NULL);
+//        e_mod_comp_shutdown();
+//        e_mod_comp_init();
+     }
+   e_config_save_queue();
+   return 1;
+}
+
+static Evas_Object *
+_basic_create_widgets(E_Config_Dialog *cfd EINA_UNUSED,
+                      Evas *evas,
+                      E_Config_Dialog_Data *cfdata)
+{
+   Evas_Object *ob,*ol, *of, *otb, *oi, *orec0, *tab;
+   E_Radio_Group *rg;
+
+   e_dialog_resizable_set(cfd->dia, 1);
+   
+   orec0 = evas_object_rectangle_add(evas);
+   evas_object_name_set(orec0, "style_shadows");
+
+   tab = e_widget_table_add(evas, 0);
+   evas_object_name_set(tab, "dia_table");
+
+   otb = e_widget_toolbook_add(evas, 48 * e_scale, 48 * e_scale);
+
+   ///////////////////////////////////////////
+   ol = e_widget_list_add(evas, 0, 0);
+   
+   ob = e_widget_check_add(evas, _("Tear-free updates (VSynced)"), &(cfdata->vsync));
+   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
+   
+   ob = e_widget_check_add(evas, _("Smooth scaling of window content"), &(cfdata->smooth_windows));
+   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
+
+   ob = e_widget_check_add(evas, _("Don't composite fullscreen windows"), &(cfdata->nocomp_fs));
+   e_widget_list_object_append(ol, ob, 1, 0, 0.5);
+   
+   of = e_widget_frametable_add(evas, _("Select default style"), 0);
+   e_widget_frametable_content_align_set(of, 0.5, 0.5);
+   oi = _style_selector(evas, &(cfdata->shadow_style));
+   e_widget_frametable_object_append(of, oi, 0, 0, 1, 1, 1, 1, 1, 1);
+   e_widget_list_object_append(ol, of, 1, 1, 0.5);
+
+   e_widget_toolbook_page_append(otb, NULL, _("General"), ol, 1, 1, 1, 1, 0.5, 0.0);
+
+   ///////////////////////////////////////////
+   ol = e_widget_list_add(evas, 0, 0);
+   rg = e_widget_radio_group_new(&(cfdata->engine));
+   ob = e_widget_radio_add(evas, _("Software"), ENGINE_SW, rg);
+   e_widget_list_object_append(ol, ob, 1, 1, 0.5);
+   if (!getenv("ECORE_X_NO_XLIB"))
+     {
+        if (ecore_evas_engine_type_supported_get(ECORE_EVAS_ENGINE_OPENGL_X11))
+          {
+             ob = e_widget_radio_add(evas, _("OpenGL"), ENGINE_GL, rg);
+             e_widget_list_object_append(ol, ob, 1, 1, 0.5);
+          }
+     }
+   ob = e_widget_label_add(evas, _("To reset compositor:"));
+   e_widget_list_object_append(ol, ob, 1, 1, 0.5);
+   ob = e_widget_label_add(evas, _("Ctrl+Alt+Shift+Home"));
+   e_widget_list_object_append(ol, ob, 1, 1, 0.5);
+   
+   e_widget_toolbook_page_append(otb, NULL, _("Rendering"), ol, 0, 0, 0, 0, 0.5, 0.0);
+
+   e_widget_toolbook_page_show(otb, 0);
+
+
+   e_widget_table_object_append(tab, otb, 0, 0, 1, 1, 1, 1, 1, 1);
+   return tab;
+}
+
+static int
+_basic_apply_data(E_Config_Dialog *cfd  __UNUSED__,
+                  E_Config_Dialog_Data *cfdata)
+{
+   if ((cfdata->lock_fps != _comp_mod->conf->lock_fps) ||
        (cfdata->smooth_windows != _comp_mod->conf->smooth_windows) ||
        (cfdata->grab != _comp_mod->conf->grab) ||
        (cfdata->keep_unmapped != _comp_mod->conf->keep_unmapped) ||
@@ -1440,7 +1621,6 @@ _basic_apply_data(E_Config_Dialog *cfd  __UNUSED__,
                }
              cfdata->match.changed = 0;
           }
-        _comp_mod->conf->use_shadow = cfdata->use_shadow;
         _comp_mod->conf->lock_fps = cfdata->lock_fps;
         _comp_mod->conf->smooth_windows = cfdata->smooth_windows;
         _comp_mod->conf->grab = cfdata->grab;
