@@ -1,17 +1,9 @@
 #ifndef E_MOD_MAIN_H
 #define E_MOD_MAIN_H
 
-#define E_TYPEDEFS 1
-#include "e_int_config_edgebindings.h"
+E_Config_Dialog *e_int_config_edgebindings(E_Container *con, const char *params __UNUSED__);
+E_Config_Dialog *e_int_config_signalbindings(E_Container *con, const char *params);
 
-#undef E_TYPEDEFS
-#include "e_int_config_edgebindings.h"
-
-EAPI extern E_Module_Api e_modapi;
-
-EAPI void *e_modapi_init     (E_Module *m);
-EAPI int   e_modapi_shutdown (E_Module *m);
-EAPI int   e_modapi_save     (E_Module *m);
 
 /**
  * @addtogroup Optional_Conf
