@@ -21,7 +21,7 @@ struct _Info
    int iw, ih;
    Eina_List *dirs;
    char *curdir;
-   DIR *dir;
+   Eina_Iterator *dir;
    Ecore_Idler *idler;
    int scans;
    int con_num, zone_num, desk_x, desk_y;
@@ -67,7 +67,7 @@ struct _Item
 static Info *global_info = NULL;
 
 static void _e_smart_reconfigure(Evas_Object *obj);
-static void _e_smart_reconfigure(Evas_Object *obj);
+static Eina_Bool _e_smart_reconfigure_do(void *data);
 static void _thumb_gen(void *data, Evas_Object *obj, void *event_info);
 static void _item_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _item_up(void *data, Evas *e, Evas_Object *obj, void *event_info);
@@ -134,6 +134,7 @@ _e_smart_reconfigure_do(void *data)
    int iw, redo = 0, changed = 0;
    static int recursion = 0;
    Evas_Coord x, y, xx, yy, ww, hh, mw, mh, ox, oy, dd;
+   Evas_Coord vw, vh;
    Evas *evas;
 
    if (!sd) return ECORE_CALLBACK_CANCEL;
@@ -157,6 +158,8 @@ _e_smart_reconfigure_do(void *data)
    mw = mh = 0;
 
    evas = evas_object_evas_get(obj);
+   evas_output_viewport_get(evas, NULL, NULL, &vw, &vh);
+
    EINA_LIST_FOREACH(sd->items, l, it)
      {
         if (x > (sd->w - ww))
@@ -216,12 +219,12 @@ _e_smart_reconfigure_do(void *data)
    if (sd->h > sd->ch) oy = (sd->h - sd->ch) / 2;
 
    EINA_LIST_FOREACH(sd->items, l, it)
-     { 
-        Evas_Coord dx, dy, vw, vh;
+     {
+        Evas_Coord dx, dy;
 
         dx = dy = 0;
         if ((sd->sx >= 0) && (sd->selmove > 0.0)
-            
+
 /*            &&
             ((it->x + it->w) > sd->cx) &&
             ((it->x) < (sd->cx + sd->w)) &&
@@ -291,7 +294,6 @@ _e_smart_reconfigure_do(void *data)
           }
         xx = sd->x - sd->cx + it->x + ox;
         yy = sd->y - sd->cy + it->y + oy;
-        evas_output_viewport_get(evas, NULL, NULL, &vw, &vh);
         if (E_INTERSECTS(xx, yy, it->w, it->h, 0, 0, vw, vh))
           {
              if (!it->have_thumb)
@@ -337,13 +339,13 @@ _e_smart_reconfigure_do(void *data)
 
                             f = e_theme_edje_file_get("base/theme/backgrounds",
                                                       "e/desktop/background");
-                            e_thumb_icon_file_set(it->image, f, 
+                            e_thumb_icon_file_set(it->image, f,
                                                   "e/desktop/background");
                          }
                        else
-                         e_thumb_icon_file_set(it->image, it->file, 
+                         e_thumb_icon_file_set(it->image, it->file,
                                                "e/desktop/background");
-                       e_thumb_icon_size_set(it->image, sd->info->iw, 
+                       e_thumb_icon_size_set(it->image, sd->info->iw,
                                              sd->info->ih);
                        evas_object_show(it->image);
 
@@ -370,7 +372,7 @@ _e_smart_reconfigure_do(void *data)
                   it->frame = NULL;
                }
              it->visible = EINA_FALSE;
-/*             
+/*
              if (it->have_thumb)
                {
                   if (it->do_thumb)
@@ -538,9 +540,9 @@ _pan_add(Evas *evas)
           NULL,
           NULL,
           NULL,
-          NULL, 
-          NULL, 
-          NULL, 
+          NULL,
+          NULL,
+          NULL,
           NULL
      };
    smart = evas_smart_class_new(&sc);
@@ -678,7 +680,7 @@ _pan_sel(Evas_Object *obj, Item *it)
 
    if (sd->selmove > 0.0) return;
    edje_object_signal_emit(it->frame, "e,state,selected", "e");
-   evas_object_raise(it->frame); 
+   evas_object_raise(it->frame);
    if (!it->selected)
      {
         Eina_List *l;
@@ -689,7 +691,7 @@ _pan_sel(Evas_Object *obj, Item *it)
              if (it2->selected) it2->selected = 0;
           }
         it->selected = EINA_TRUE;
-        if (sd->info->bg_file) free(sd->info->bg_file);
+        free(sd->info->bg_file);
         evas_object_hide(sd->info->mini);
         if (it->file)
           {
@@ -708,7 +710,7 @@ _pan_sel(Evas_Object *obj, Item *it)
                   if (p) *p = 0;
                }
              edje_object_part_text_set(sd->info->bg, "e.text.filename", name);
-             if (name) free(name);
+             free(name);
           }
         else
           {
@@ -718,7 +720,7 @@ _pan_sel(Evas_Object *obj, Item *it)
              edje_object_file_set(sd->info->mini, f, "e/desktop/background");
              sd->info->use_theme_bg = 1;
              sd->info->bg_file = NULL;
-             edje_object_part_text_set(sd->info->bg, "e.text.filename", 
+             edje_object_part_text_set(sd->info->bg, "e.text.filename",
                                        _("Theme Wallpaper"));
           }
         evas_object_show(sd->info->mini);
@@ -825,15 +827,15 @@ _thumb_gen(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
      }
 }
 
-static void         
+static void
 _item_down(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
 //   Evas_Event_Mouse_Down *ev = event_info;
 //   Item *it = data;
 //   _pan_sel(it->obj, it);
 }
-    
-static void         
+
+static void
 _item_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Mouse_Up *ev = event_info;
@@ -849,7 +851,7 @@ _item_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *even
           }
      }
 }
-    
+
 static void
 _pan_file_add(Evas_Object *obj, const char *file, Eina_Bool remote, Eina_Bool theme)
 {
@@ -911,7 +913,7 @@ _pan_file_add(Evas_Object *obj, const char *file, Eina_Bool remote, Eina_Bool th
         if (sd->info->use_theme_bg)
           {
              _pan_hilight(it->obj, it);
-             edje_object_part_text_set(sd->info->bg, "e.text.filename", 
+             edje_object_part_text_set(sd->info->bg, "e.text.filename",
                                        _("Theme Wallpaper"));
           }
      }
@@ -945,7 +947,7 @@ _pan_file_add(Evas_Object *obj, const char *file, Eina_Bool remote, Eina_Bool th
                        if (p) *p = 0;
                     }
                   edje_object_part_text_set(sd->info->bg, "e.text.filename", name);
-                  if (name) free(name);
+                  free(name);
                }
           }
      }
@@ -977,7 +979,7 @@ _bg_clicked(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUS
 }
 
 static void
-_ok(void *data, void *data2 __UNUSED__)
+_apply(void *data, void *data2 __UNUSED__)
 {
    Info *info = data;
 
@@ -1000,7 +1002,7 @@ _ok(void *data, void *data2 __UNUSED__)
      {
         /* specific desk */
         e_bg_del(info->con_num, info->zone_num, info->desk_x, info->desk_y);
-        e_bg_add(info->con_num, info->zone_num, info->desk_x, info->desk_y, 
+        e_bg_add(info->con_num, info->zone_num, info->desk_x, info->desk_y,
                  info->bg_file);
      }
    else
@@ -1020,9 +1022,21 @@ _ok(void *data, void *data2 __UNUSED__)
      }
    e_bg_update();
    e_config_save_queue();
+}
+
+static void
+_close(void *data __UNUSED__, void *data2 __UNUSED__)
+{
    wp_conf_hide();
 }
 
+static void
+_ok(void *data, void *data2 __UNUSED__)
+{
+  _apply(data, data2);
+  wp_conf_hide();
+}
+
 static void
 _wp_add(void *data, void *data2 __UNUSED__)
 {
@@ -1050,8 +1064,7 @@ _wp_changed(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__
 static Eina_Bool
 _idler(void *data)
 {
-   struct dirent *dp;
-   char buf[PATH_MAX];
+   Eina_File_Direct_Info *st;
    Info *info = data;
 
    if (!info->dir)
@@ -1059,29 +1072,24 @@ _idler(void *data)
         info->idler = NULL;
         return ECORE_CALLBACK_CANCEL;
      }
-   dp = readdir(info->dir);
-   if (!dp)
+   if (!eina_iterator_next(info->dir, (void**) &st))
      {
-        free(info->curdir);
-        info->curdir = NULL;
-        closedir(info->dir);
+        E_FREE(info->curdir);
+        eina_iterator_free(info->dir);
         info->dir = NULL;
         info->idler = NULL;
         _scan(info);
         return ECORE_CALLBACK_CANCEL;
      }
-   if ((!strcmp(dp->d_name, ".")) || (!strcmp(dp->d_name, "..")))
+   if ((st->path[st->name_start]) == '.')
      return ECORE_CALLBACK_RENEW;
-   if (dp->d_name[0] == '.')
-     return ECORE_CALLBACK_RENEW;
-   snprintf(buf, sizeof(buf), "%s/%s", info->curdir, dp->d_name);
-   if (ecore_file_is_dir(buf))
+   if (st->type == EINA_FILE_DIR)
      {
-        info->dirs = eina_list_append(info->dirs, strdup(buf));
+        info->dirs = eina_list_append(info->dirs, strdup(st->path));
         return ECORE_CALLBACK_RENEW;
      }
    info->scans++;
-   _pan_file_add(info->span, buf, 0, 0);
+   _pan_file_add(info->span, st->path, 0, 0);
 
    e_util_wakeup();
    return ECORE_CALLBACK_RENEW;
@@ -1096,13 +1104,13 @@ _scan(Info *info)
           {
              info->scans = 0;
              edje_object_signal_emit(info->bg, "e,state,busy,on", "e");
-             edje_object_part_text_set(info->bg, "e.text.busy_label", 
+             edje_object_part_text_set(info->bg, "e.text.busy_label",
                                        _("Loading files..."));
           }
-        if (info->curdir) free(info->curdir);
+        free(info->curdir);
         info->curdir = info->dirs->data;
         info->dirs = eina_list_remove_list(info->dirs, info->dirs);
-        if (!info->dir) info->dir = opendir(info->curdir);
+        if (!info->dir) info->dir = eina_file_stat_ls(info->curdir);
         info->idler = ecore_idler_add(_idler, info);
      }
 }
@@ -1118,7 +1126,7 @@ wp_browser_new(E_Container *con)
    Evas_Coord mw, mh;
    Evas_Object *o, *o2, *ob;
    E_Radio_Group *rg;
-   char buf[PATH_MAX];   
+   char buf[PATH_MAX];
 
    info = calloc(1, sizeof(Info));
    if (!info) return NULL;
@@ -1153,8 +1161,7 @@ wp_browser_new(E_Container *con)
    win = e_win_new(con);
    if (!win)
      {
-        free(info);
-        info = NULL;
+        E_FREE(info);
         return NULL;
      }
    info->win = win;
@@ -1177,14 +1184,26 @@ wp_browser_new(E_Container *con)
    edje_object_signal_callback_add(info->bg, "e,action,click", "e",
                                    _bg_clicked, info);
 
-   // ok button
    info->box = e_widget_list_add(info->win->evas, 1, 1);
 
-   info->button = e_widget_button_add(info->win->evas, _("OK"), NULL, 
+   // ok button
+   info->button = e_widget_button_add(info->win->evas, _("OK"), NULL,
                                       _ok, info, NULL);
    evas_object_show(info->button);
    e_widget_list_object_append(info->box, info->button, 1, 0, 0.5);
 
+   // apply button
+   info->button = e_widget_button_add(info->win->evas, _("Apply"), NULL,
+                                      _apply, info, NULL);
+   evas_object_show(info->button);
+   e_widget_list_object_append(info->box, info->button, 1, 0, 0.5);
+
+   // close button
+   info->button = e_widget_button_add(info->win->evas, _("Close"), NULL,
+                                      _close, info, NULL);
+   evas_object_show(info->button);
+   e_widget_list_object_append(info->box, info->button, 1, 0, 0.5);
+
    e_widget_size_min_get(info->box, &mw, &mh);
    edje_extern_object_min_size_set(info->box, mw, mh);
    edje_object_part_swallow(info->bg, "e.swallow.buttons", info->box);
@@ -1193,7 +1212,7 @@ wp_browser_new(E_Container *con)
    // preview
    info->preview = e_livethumb_add(info->win->evas);
    e_livethumb_vsize_set(info->preview, zone->w, zone->h);
-   edje_extern_object_aspect_set(info->preview, EDJE_ASPECT_CONTROL_NEITHER, 
+   edje_extern_object_aspect_set(info->preview, EDJE_ASPECT_CONTROL_NEITHER,
                                  zone->w, zone->h);
    edje_object_part_swallow(info->bg, "e.swallow.preview", info->preview);
    evas_object_show(info->preview);
@@ -1220,7 +1239,7 @@ wp_browser_new(E_Container *con)
    e_scrollframe_custom_theme_set(info->sframe, "base/theme/widgets",
                                   "e/conf/wallpaper/main/scrollframe");
    e_scrollframe_extern_pan_set(info->sframe, info->span,
-                                _pan_set, _pan_get, _pan_max_get, 
+                                _pan_set, _pan_get, _pan_max_get,
                                 _pan_child_size_get);
    edje_object_part_swallow(info->bg, "e.swallow.list", info->sframe);
    evas_object_show(info->sframe);
@@ -1245,7 +1264,7 @@ wp_browser_new(E_Container *con)
    o2 = e_widget_radio_add(info->win->evas, _("This Screen"), 2, rg);
    evas_object_smart_callback_add(o2, "changed", _wp_changed, info);
    e_widget_list_object_append(o, o2, 1, 0, 0.5);
-   if (!(e_util_container_zone_number_get(0, 1) || 
+   if (!(e_util_container_zone_number_get(0, 1) ||
          (e_util_container_zone_number_get(1, 0))))
      e_widget_disabled_set(o2, EINA_TRUE);
    evas_object_show(o2);
@@ -1255,12 +1274,12 @@ wp_browser_new(E_Container *con)
 
    o = e_widget_list_add(info->win->evas, 1, 0);
 
-   o2 =  e_widget_button_add(info->win->evas, _("Add"), NULL, 
+   o2 =  e_widget_button_add(info->win->evas, _("Add"), NULL,
                              _wp_add, info, NULL);
    e_widget_list_object_append(o, o2, 1, 0, 0.5);
    evas_object_show(o2);
 
-   o2 =  e_widget_button_add(info->win->evas, _("Delete"), NULL, 
+   o2 =  e_widget_button_add(info->win->evas, _("Delete"), NULL,
                              _wp_delete, info, NULL);
    e_widget_list_object_append(o, o2, 1, 0, 0.5);
    evas_object_show(o2);
@@ -1300,15 +1319,14 @@ wp_broser_free(Info *info)
 
    if (!info) return;
    e_object_del(E_OBJECT(info->win));
-   if (info->dir) closedir(info->dir);
+   if (info->dir) eina_iterator_free(info->dir);
    free(info->bg_file);
    free(info->curdir);
-   EINA_LIST_FREE(info->dirs, s) 
+   EINA_LIST_FREE(info->dirs, s)
      free(s);
    if (info->idler) ecore_idler_del(info->idler);
    // del other stuff
-   free(info);
-   info = NULL;
+   E_FREE(info);
 }
 
 E_Config_Dialog *
