@@ -1,17 +1,15 @@
 #include "e_mod_main.h"
 
-
 typedef struct _Plugin Plugin;
 
 struct _Plugin
 {
-  Evry_Plugin base;
+   Evry_Plugin base;
 
-  Eina_List *plugins;
+   Eina_List  *plugins;
 };
 
 static Eina_List *plugins = NULL;
-static const char _module_icon[] = "application-other";
 static Evry_Type COLLECTION_PLUGIN;
 static Plugin_Config plugin_config;
 
@@ -34,12 +32,12 @@ _browse(Evry_Plugin *plugin, const Evry_Item *item)
 
    if (pp->begin && (inst = pp->begin(pp, NULL)))
      {
-	if (!strcmp(plugin->name, "Plugins"))
-	  inst->config = &plugin_config;
-	else
-	  inst->config = pc;
+        if (!strcmp(plugin->name, "Plugins"))
+          inst->config = &plugin_config;
+        else
+          inst->config = pc;
 
-	return inst;
+        return inst;
      }
 
    return NULL;
@@ -53,22 +51,22 @@ _add_item(Plugin *p, Plugin_Config *pc)
 
    if (pc->enabled && (pp = evry_plugin_find(pc->name)))
      {
-	pc->plugin = pp;
+        pc->plugin = pp;
 
-	GET_ITEM(itp, pp);
-	it = EVRY_ITEM_NEW(Evry_Item, EVRY_PLUGIN(p), itp->label, NULL, NULL);
-	if (itp->icon) it->icon = eina_stringshare_ref(itp->icon);
-	it->icon_get = itp->icon_get;
-	it->data = pc;
-	it->browseable = EINA_TRUE;
-	it->detail = eina_stringshare_ref(EVRY_ITEM(p)->label);
-	p->plugins = eina_list_append(p->plugins, it);
+        GET_ITEM(itp, pp);
+        it = EVRY_ITEM_NEW(Evry_Item, EVRY_PLUGIN(p), itp->label, NULL, NULL);
+        if (itp->icon) it->icon = eina_stringshare_ref(itp->icon);
+        it->icon_get = itp->icon_get;
+        it->data = pc;
+        it->browseable = EINA_TRUE;
+        it->detail = eina_stringshare_ref(EVRY_ITEM(p)->label);
+        p->plugins = eina_list_append(p->plugins, it);
      }
    return it;
 }
 
 static Evry_Plugin *
-_begin(Evry_Plugin *plugin, const Evry_Item *item)
+_begin(Evry_Plugin *plugin, const Evry_Item *item __UNUSED__)
 {
    Plugin_Config *pc;
    Eina_List *l;
@@ -76,14 +74,14 @@ _begin(Evry_Plugin *plugin, const Evry_Item *item)
 
    EVRY_PLUGIN_INSTANCE(p, plugin);
 
-   EINA_LIST_FOREACH(plugin->config->plugins, l, pc)
+   EINA_LIST_FOREACH (plugin->config->plugins, l, pc)
      _add_item(p, pc);
 
    return EVRY_PLUGIN(p);
 }
 
 static Evry_Plugin *
-_begin_all(Evry_Plugin *plugin, const Evry_Item *item)
+_begin_all(Evry_Plugin *plugin, const Evry_Item *item __UNUSED__)
 {
    Plugin_Config *pc;
    Eina_List *l;
@@ -91,15 +89,15 @@ _begin_all(Evry_Plugin *plugin, const Evry_Item *item)
 
    EVRY_PLUGIN_INSTANCE(p, plugin);
 
-   EINA_LIST_FOREACH(evry_conf->conf_subjects, l, pc)
+   EINA_LIST_FOREACH (evry_conf->conf_subjects, l, pc)
      {
-	if (!strcmp(pc->name, "All") ||
-	    !strcmp(pc->name, "Actions") ||
-	    !strcmp(pc->name, "Calculator") ||
-	    !strcmp(pc->name, "Plugins"))
-	  continue;
+        if (!strcmp(pc->name, "All") ||
+            !strcmp(pc->name, "Actions") ||
+            !strcmp(pc->name, "Calculator") ||
+            !strcmp(pc->name, "Plugins"))
+          continue;
 
-     _add_item(p, pc);
+        _add_item(p, pc);
      }
 
    return EVRY_PLUGIN(p);
@@ -114,7 +112,7 @@ _finish(Evry_Plugin *plugin)
 
    EVRY_PLUGIN_ITEMS_CLEAR(p);
 
-   EINA_LIST_FREE(p->plugins, it)
+   EINA_LIST_FREE (p->plugins, it)
      EVRY_ITEM_FREE(it);
 
    E_FREE(p);
@@ -129,7 +127,7 @@ _fetch(Evry_Plugin *plugin, const char *input)
 
    EVRY_PLUGIN_ITEMS_ADD(p, p->plugins, input, 1, 0);
 
-   return !!(plugin->items);
+   return EVRY_PLUGIN_HAS_ITEMS(p);
 }
 
 static Evry_Plugin *
@@ -139,17 +137,16 @@ _add_plugin(const char *name)
    char path[4096];
    char title[4096];
 
-   p = EVRY_PLUGIN_NEW(Evry_Plugin, N_(name),
-		       _module_icon, COLLECTION_PLUGIN,
-		       _begin, _finish, _fetch, NULL);
+   p = EVRY_PLUGIN_BASE(name, NULL, COLLECTION_PLUGIN,
+                        _begin, _finish, _fetch);
    p->browse = &_browse;
 
-   snprintf(path, sizeof(path), "extensions/everything-%s", p->name);
+   snprintf(path, sizeof(path), "launcher/everything-%s", p->name);
 
-   snprintf(title, sizeof(title), "Everything %s", p->name);
+   snprintf(title, sizeof(title), "%s: %s", _("Everything Plugin"), p->base.label);
 
-   e_configure_registry_item_add
-     (path, 110, title, NULL, NULL/*icon*/, evry_collection_conf_dialog);
+   e_configure_registry_item_params_add
+     (path, 110, title, NULL, p->base.icon, evry_collection_conf_dialog, p->name);
 
    p->config_path = eina_stringshare_add(path);
 
@@ -172,26 +169,26 @@ evry_plug_collection_init(void)
 
    COLLECTION_PLUGIN = evry_type_register("COLLECTION_PLUGIN");
 
-   e_configure_registry_category_add
-     ("extensions", 80, _("Extensions"), NULL, "preferences-extensions");
-
    p = _add_plugin("Plugins");
    p->begin = &_begin_all;
+   EVRY_ITEM_ICON_SET(p, "preferences-plugin");
+
    if (evry_plugin_register(p, EVRY_PLUGIN_SUBJECT, 100))
      {
-	p->config->aggregate = EINA_TRUE;
-	p->config->top_level = EINA_TRUE;
-	p->config->view_mode = VIEW_MODE_THUMB;
+        p->config->aggregate = EINA_TRUE;
+        p->config->top_level = EINA_TRUE;
+        p->config->view_mode = VIEW_MODE_THUMB;
      }
 
-   EINA_LIST_FOREACH(evry_conf->collections, l, pc)
+   EINA_LIST_FOREACH (evry_conf->collections, l, pc)
      {
-	p = _add_plugin(pc->name); 
-	p->config = pc;
-	pc->plugin = p;
+        p = _add_plugin(pc->name);
+        EVRY_ITEM_ICON_SET(p, "start-here");
+        p->config = pc;
+        pc->plugin = p;
 
-	if (evry_plugin_register(p, EVRY_PLUGIN_SUBJECT, 1))
-	  p->config->aggregate = EINA_FALSE;
+        if (evry_plugin_register(p, EVRY_PLUGIN_SUBJECT, 1))
+          p->config->aggregate = EINA_FALSE;
      }
 
    return EINA_TRUE;
@@ -202,14 +199,15 @@ evry_plug_collection_shutdown(void)
 {
    Evry_Plugin *p;
 
-   EINA_LIST_FREE(plugins, p)
+   EINA_LIST_FREE (plugins, p)
      {
-	if (p->config_path)
-	  {
-	     e_configure_registry_item_del(p->config_path);
-	     eina_stringshare_del(p->config_path);
-	  }
+        if (p->config_path)
+          {
+             e_configure_registry_item_del(p->config_path);
+             eina_stringshare_del(p->config_path);
+          }
 
-	EVRY_PLUGIN_FREE(p);
+        EVRY_PLUGIN_FREE(p);
      }
 }
+
