@@ -247,7 +247,8 @@ struct _E_Illume_Config
         struct 
           {
              int duration;
-             /**< integer specifying the amount of time it takes for an 
+	     int resize_before; /* resize app window before hide animation */
+	     /**< integer specifying the amount of time it takes for an 
               * animation to complete. */
           } vkbd, quickpanel;
      } animation;
@@ -346,6 +347,7 @@ EAPI Eina_Bool e_illume_border_is_qt_frame(E_Border *bd);
 EAPI Eina_Bool e_illume_border_is_fullscreen(E_Border *bd);
 EAPI Eina_Bool e_illume_border_is_conformant(E_Border *bd);
 EAPI Eina_Bool e_illume_border_is_quickpanel(E_Border *bd);
+EAPI Eina_Bool e_illume_border_is_fixed_size(E_Border *bd);
 
 EAPI void e_illume_border_min_get(E_Border *bd, int *w, int *h);
 EAPI E_Border *e_illume_border_at_xy_get(E_Zone *zone, int x, int y);
