@@ -1,19 +1,19 @@
 /* Setup of default icon theme */
-#include "e.h"
-#include "e_mod_main.h"
+#include "e_wizard.h"
 
 EAPI int
-wizard_page_init(E_Wizard_Page *pg __UNUSED__)
+wizard_page_init(E_Wizard_Page *pg __UNUSED__, Eina_Bool *need_xdg_desktops __UNUSED__, Eina_Bool *need_xdg_icons)
 {
+   *need_xdg_icons = EINA_TRUE;
    return 1;
 }
-
+/*
 EAPI int
 wizard_page_shutdown(E_Wizard_Page *pg __UNUSED__)
 {
    return 1;
 }
-
+*/
 EAPI int
 wizard_page_show(E_Wizard_Page *pg __UNUSED__)
 {
@@ -22,17 +22,17 @@ wizard_page_show(E_Wizard_Page *pg __UNUSED__)
    int i;
    const char *selected = NULL;
    const char *search[] =
-     {
-        "gnome",
-        "Humanity",
-        "Humanity-Dark",
-        "ubuntu-mono-light",
-        "ubuntu-mono-dark",
-        "ubuntu-mono-light",
-        "unity-icon-theme",
-        NULL
-     };
-   
+   {
+      "gnome",
+      "Humanity",
+      "Humanity-Dark",
+      "ubuntu-mono-light",
+      "ubuntu-mono-dark",
+      "ubuntu-mono-light",
+      "unity-icon-theme",
+      NULL
+   };
+
    if (!themes) return 0;
    for (i = 0; search[i]; i++)
      {
@@ -54,7 +54,7 @@ done:
    eina_list_free(themes);
    return 0; /* 1 == show ui, and wait for user, 0 == just continue */
 }
-
+/*
 EAPI int
 wizard_page_hide(E_Wizard_Page *pg __UNUSED__)
 {
@@ -66,3 +66,4 @@ wizard_page_apply(E_Wizard_Page *pg __UNUSED__)
 {
    return 1;
 }
+*/
