@@ -1,9 +1,9 @@
 /* Delete previous copy of config profile and save new one */
-#include "e.h"
-#include "e_mod_main.h"
+#include "e_wizard.h"
 
+#if 0
 EAPI int
-wizard_page_init(E_Wizard_Page *pg __UNUSED__)
+wizard_page_init(E_Wizard_Page *pg __UNUSED__, Eina_Bool *need_xdg_desktops __UNUSED__, Eina_Bool *need_xdg_icons __UNUSED__)
 {
    return 1;
 }
@@ -25,7 +25,7 @@ wizard_page_hide(E_Wizard_Page *pg __UNUSED__)
 {
    return 1;
 }
-
+#endif
 EAPI int
 wizard_page_apply(E_Wizard_Page *pg __UNUSED__)
 {
@@ -37,3 +37,4 @@ wizard_page_apply(E_Wizard_Page *pg __UNUSED__)
    e_sys_action_do(E_SYS_RESTART, NULL);
    return 1;
 }
+
